#!/usr/bin/env python3
#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

# pylint: disable=broad-except,found-_-in-module-class

import argparse
import os
import sys
import warnings
import tempfile
import shutil
from pykickstart.i18n import _
from pykickstart.errors import KickstartError, KickstartParseError, KickstartVersionError,\
    KickstartParseWarning, KickstartDeprecationWarning
from pykickstart.load import load_to_file
from pykickstart.parser import KickstartParser, preprocessKickstart
from pykickstart.version import DEVEL, makeVersion, versionMap

def cleanup(dest, fn=None, exitval=1):
    shutil.rmtree(dest)

    # Don't care if this file doesn't exist.
    if fn is not None:
        try:
            os.remove(fn)
        except FileNotFoundError:
            pass

    return exitval

def main(argv):
    op = argparse.ArgumentParser(usage="%(prog)s [options] ksfile", add_help=False)
    op.add_argument("ksfile", nargs="?",
                    help=_("filename or URL to read from"))
    op.add_argument("-e", "--firsterror", dest="firsterror", action="store_true",
                    default=False, help=_("halt after the first error or warning"))
    op.add_argument("-i", "--followincludes", dest="followincludes",
                    action="store_true", default=False,
                    help=_("parse include files when %%include is seen"))
    op.add_argument("-l", "--listversions", dest="listversions", action="store_true",
                    default=False,
                    help=_("list the available versions of kickstart syntax"))
    op.add_argument("-v", "--version", dest="version", default=DEVEL,
                    help=_("version of kickstart syntax to validate against"))
    op.add_argument("-h", "--help", dest="help", action="store_true", default=False,
                    help=_("show this help message and exit"))

    opts = op.parse_args(argv)

    # parse --help manually b/c we don't want to sys.exit before the
    # tests have finished
    if opts.help:
        return (0, op.format_help().split("\n"))

    if opts.listversions:
        versions = []
        for key in sorted(versionMap.keys()):
            versions.append(key)
        return (0, versions)

    if not opts.ksfile:
        return (1, op.format_usage().split("\n"))

    destdir = tempfile.mkdtemp("", "ksvalidator-tmp-", "/tmp")
    try:
        f = load_to_file(opts.ksfile, "%s/ks.cfg" % destdir)
    except KickstartError as e:
        return (cleanup(destdir),
                [_("Error reading %(filename)s:\n%(version)s") % {"filename": opts.ksfile, "version": e}])

    try:
        handler = makeVersion(opts.version)
    except KickstartVersionError:
        return (cleanup(destdir),
                [_("The version %s is not supported by pykickstart") % opts.version])

    ksparser = KickstartParser(handler, followIncludes=opts.followincludes,
                               errorsAreFatal=opts.firsterror)

    # turn kickstart parse warnings into errors
    warnings.filterwarnings(action="error", category=KickstartParseWarning)

    processedFile = None

    try:
        processedFile = preprocessKickstart(f)
        ksparser.readKickstart(processedFile)
        return (cleanup(destdir, processedFile, exitval=ksparser.errorsCount), [])
    except KickstartDeprecationWarning as err:
        return (cleanup(destdir, processedFile),
                [_("File uses a deprecated option or command.\n%s") % err])
    except KickstartParseError as err:
        return (cleanup(destdir, processedFile), [str(err)])
    except KickstartError:
        return (cleanup(destdir, processedFile),
                [_("General kickstart error in input file")])
    except Exception as e:                                      # pylint: disable=broad-except
        return (cleanup(destdir, processedFile),
                [_("General error in input file:  %s") % e])

if __name__ == "__main__":
    retval, messages = main(sys.argv[1:])
    for msg in messages:
        print(msg)
    sys.exit(retval)



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2015 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.version import F23
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class F23_ReqPart(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.reqpart = kwargs.get("reqpart", False)
        self.addBoot = kwargs.get("addBoot", False)

        self.op = self._getParser()

    def _getArgsAsStr(self):
        retval = ""

        if self.addBoot:
            retval += " --add-boot"

        return retval

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if self.reqpart:
            retval += "reqpart%s\n" % self._getArgsAsStr()
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="reqpart", description="""
                            Automatically create partitions required by your
                            hardware platform. These include a ``/boot/efi``
                            for x86_64 and Aarch64 systems with UEFI firmware,
                            ``biosboot`` for x86_64 systems with BIOS firmware
                            and GPT, and ``PRePBoot`` for IBM Power Systems.

                            Note: This command can not be used together with
                            ``autopart``, because ``autopart`` does the same
                            and creates other partitions or logical volumes
                            such as ``/`` and ``swap`` on top. In contrast with
                            ``autopart``, this command only creates
                            platform-specific partitions and leaves the rest of
                            the drive empty, allowing you to create a custom
                            layout.""", version=F23)
        op.add_argument("--add-boot", action="store_true", version=F23,
                        dest="addBoot", default=False, help="""
                        Create a separate ``/boot`` partition in addition to the
                        platform-specific partition created by the base command.
                        """)
        return op

    def parse(self, args):
        # Using reqpart and autopart at the same time is not allowed.
        if self.handler.autopart.seen:
            errorMsg = _("The %s and reqpart commands can't be used at the same time") % \
                         "autopart"
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        # the same applies to the 'mount' command
        if hasattr(self.handler, "mount") and self.handler.mount.seen:
            errorMsg = _("The mount and reqpart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        self.reqpart = True
        return self

class RHEL7_ReqPart(F23_ReqPart):
    pass



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.group import F12_GroupData
from pykickstart.version import F12

class Group_TestCase(unittest.TestCase):
    def runTest(self):
        # test that new objects are always equal
        data1 = F12_GroupData()
        data2 = F12_GroupData()
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['name']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test-group')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class F12_TestCase(CommandTest):
    command = "group"

    def runTest(self):
        # pass
        self.assert_parse("group --name=test", "group --name=test\n")
        self.assert_parse("group --name=test --gid=1000", "group --name=test --gid=1000\n")

        self.assertFalse(self.assert_parse("group --name=test") is None)
        self.assertTrue(self.assert_parse("group --name=testA") != self.assert_parse("group --name=testB"))
        self.assertFalse(self.assert_parse("group --name=testA") == self.assert_parse("group --name=testB"))

        # fail
        # missing required option --name
        self.assert_parse_error("group")
        # --name requires an argument
        self.assert_parse_error("group --name")
        # --gid requires int argument
        self.assert_parse_error("group --name=test --uid=id")
        # unknown option
        self.assert_parse_error("group --name=test --unknown=value")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.groupList = ["test"]
        self.assertEqual(cmd.__str__(), "test")

        gd = self.handler().GroupData()
        gd.name = ""
        self.assertEqual(gd.__str__(), "group\n")

class F12_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F12

    def runTest(self):
        self.assert_parse("""
group --name=test
group --name=othertest""")

        self.assert_parse_error("""
group --name=test --gid=1000
group --name=test --gid=1010""", KickstartParseWarning)

if __name__ == "__main__":
    unittest.main()



import unittest
from tests.baseclass import CommandTest

class F7_TestCase(CommandTest):
    command = "updates"

    def runTest(self):
        # pass
        self.assert_parse("updates", "updates\n")
        self.assert_parse("updates deliciouscheeses", "updates deliciouscheeses\n")

        # fail
        self.assert_parse_error("updates cheese crackers")
        self.assert_parse_error("updates --bogus-option")

if __name__ == "__main__":
    unittest.main()



import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.lang import FC3_Lang, F19_Lang

class Lang_TestCase(unittest.TestCase):
    def runTest(self):
        self.assertEqual(FC3_Lang().__str__(), '')
        self.assertEqual(F19_Lang().__str__(), '')


class FC3_TestCase(CommandTest):
    command = "lang"

    def runTest(self):
        # pass
        self.assert_parse("lang en_US", "lang en_US\n")

        # fail
        # Fail if less than or more than one argument is specified
        self.assert_parse_error("lang")
        self.assert_parse_error("lang en_US en_CA")
        self.assert_parse_error("lang --whatever")


class F19_TestCase(FC3_TestCase):
    def runTest(self):

        # pass
        self.assert_parse("lang en_US")
        self.assert_parse("lang en_US --addsupport=cs_CZ",
                          "lang en_US --addsupport=cs_CZ\n")
        self.assert_parse("lang en_US --addsupport=sr_RS.UTF-8@latin",
                          "lang en_US --addsupport=sr_RS.UTF-8@latin\n")
        self.assert_parse("lang en_US --addsupport=cs_CZ,fr_FR",
                          "lang en_US --addsupport=cs_CZ,fr_FR\n")

        # fail
        self.assert_parse_error("lang --bogus-option")
        # Fail if less than or more than one argument is specified
        self.assert_parse_error("lang")
        self.assert_parse_error("lang en_US en_CA")
        self.assert_parse_error("lang --addsupport=en_US")
        self.assert_parse_error("lang --addsupport=,bg_BG")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3
from pykickstart.base import KickstartCommand
from pykickstart.constants import SELINUX_DISABLED, SELINUX_ENFORCING, SELINUX_PERMISSIVE
from pykickstart.options import KSOptionParser

class FC3_SELinux(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.selinux = kwargs.get("selinux", None)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not retval and self.selinux is None:
            return ""

        retval += "# SELinux configuration\n"

        if self.selinux == SELINUX_DISABLED:
            retval += "selinux --disabled\n"
        elif self.selinux == SELINUX_ENFORCING:
            retval += "selinux --enforcing\n"
        elif self.selinux == SELINUX_PERMISSIVE:
            retval += "selinux --permissive\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="selinux", description="""
                            Sets the state of SELinux on the installed system.
                            SELinux defaults to enforcing in anaconda.""",
                            epilog="""
                            Only one of ``--disabled``, ``--enabled`` or
                            ``--permissive`` must be specified!""",
                            version=FC3)
        op.add_argument("--disabled", dest="selinux", version=FC3,
                        action="store_const", const=SELINUX_DISABLED,
                        help="If this is present, SELinux is disabled.")
        op.add_argument("--enforcing", dest="selinux", version=FC3,
                        action="store_const", const=SELINUX_ENFORCING,
                        help="""
                        If this is present, SELinux is set to enforcing mode.
                        """)
        op.add_argument("--permissive", dest="selinux", version=FC3,
                        action="store_const", const=SELINUX_PERMISSIVE,
                        help="""
                        If this is present, SELinux is enabled, but only logs
                        things that would be denied in enforcing mode.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self



#
# Copyright (C) 2017  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#

import unittest
from tests.baseclass import CommandTest

from pykickstart.errors import KickstartParseError

class F27_TestCase(CommandTest):
    command = "mount"

    def runTest(self):
        self.assert_parse("mount /dev/sda1 /boot", "mount /dev/sda1 /boot\n")
        self.assert_parse("mount /dev/sda1 /boot --reformat", "mount /dev/sda1 /boot --reformat\n")
        self.assert_parse("mount /dev/sda1 /boot --reformat=xfs", "mount /dev/sda1 /boot --reformat=xfs\n")
        self.assert_parse("mount /dev/sda1 /boot --reformat=xfs --mkfsoptions=\"-L BOOT\"",
                          "mount /dev/sda1 /boot --reformat=xfs --mkfsoptions=\"-L BOOT\"\n")
        self.assert_parse("mount /dev/sda1 /boot --reformat=xfs --mountoptions=user",
                          "mount /dev/sda1 /boot --reformat=xfs --mountoptions=\"user\"\n")
        self.assert_parse("mount /dev/sda3 none  --reformat=swap", "mount /dev/sda3 none --reformat=swap\n")

        self.assert_parse_error("mount /dev/sda1", KickstartParseError)
        self.assert_parse_error("mount /boot", KickstartParseError)
        self.assert_parse_error("mount /dev/sda1 /boot --mkfsoptions=\"-L BOOT\"", KickstartParseError)

if __name__ == "__main__":
    unittest.main()



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.mouse import RHEL3_Mouse

class Mouse_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = RHEL3_Mouse()
        self.assertEqual(cmd.emulthree, False)

class RHEL3_TestCase(CommandTest):
    command = "mouse"

    def runTest(self):
        # pass
        self.assert_parse("mouse jerry", "mouse jerry\n")
        self.assert_parse("mouse --device=/dev/mice --emulthree jerry", "mouse --device=/dev/mice --emulthree jerry\n")
        self.assert_parse("mouse jerry --device=/dev/mice", "mouse --device=/dev/mice jerry\n")
        self.assert_parse("mouse jerry --emulthree", "mouse --emulthree jerry\n")

        # fail
        # empty
        self.assert_parse_error("mouse")
        # multiple mice specified
        self.assert_parse_error("mouse tom and jerry")
        # unknown option
        self.assert_parse_error("mouse --bad-flag")
        self.assert_parse_error("mouse jerry --bad-flag")
        # --device requires argument
        self.assert_parse_error("mouse jerry --device")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.mouse = False
        self.assertEqual(cmd.__str__(), "")

if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2013  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#
from pykickstart.version import F20
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class F20_Eula(KickstartCommand):
    """The 'eula' kickstart command"""

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.agreed = kwargs.get("agreed", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.agreed:
            retval += "# License agreement\n"
            retval += "eula %s\n" % self._getArgsAsStr()

        return retval

    def _getArgsAsStr(self):
        if self.agreed:
            return "--agreed"
        else:
            return ""

    def _getParser(self):
        op = KSOptionParser(prog="eula", version=F20, description="""
                            Automatically accept Red Hat's EULA""")
        # people would struggle remembering the exact word
        op.add_argument("--agreed", "--agree", "--accepted", "--accept",
                        dest="agreed", action="store_true", default=False,
                        version=F20, help="Accept the EULA. This is mandatory option!")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        self.set_to_self(ns)

        if extra:
            raise KickstartParseError(_("Kickstart command %s does not take any arguments") % "eula", lineno=self.lineno)

        if not self.agreed:
            raise KickstartParseError(_("Kickstart command eula expects the --agreed option"), lineno=self.lineno)

        return self



import unittest
from tests.baseclass import ParserTest

class Packages_Seen_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%packages
packageA
%end
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertTrue(self.handler.packages.seen)

class Packages_Not_Seen_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
autopart
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertFalse(self.handler.packages.seen)

class Commands_Seen_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
bootloader --location=none
part / --size=10000 --fstype=ext4
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertFalse(self.handler.autopart.seen)
        self.assertTrue(self.handler.bootloader.seen)
        self.assertTrue(self.handler.partition.seen)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.errors import KickstartParseWarning
from pykickstart.version import FC6, F8, F12, F19, F24
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser, commaSplit

import warnings
import six
from pykickstart.i18n import _

class FC6_UserData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.groups = kwargs.get("groups", [])
        self.homedir = kwargs.get("homedir", "")
        self.isCrypted = kwargs.get("isCrypted", False)
        self.name = kwargs.get("name", "")
        self.password = kwargs.get("password", "")
        self.shell = kwargs.get("shell", "")
        self.uid = kwargs.get("uid", None)

    def __eq__(self, y):
        if not y:
            return False

        return self.name == y.name

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)

        args = self._getArgsAsStr()
        if args:
            retval += "user%s\n" % args

        return retval

    def _getArgsAsStr(self):
        retval = ""

        if self.groups:
            retval += " --groups=%s" % ",".join(self.groups)
        if self.homedir:
            retval += " --homedir=%s" % self.homedir
        if self.name:
            retval += " --name=%s" % self.name
        if self.password:
            if isinstance(self.password, six.binary_type) and b'#' in self.password:
                retval += " --password=\"%s\"" % self.password
            elif not isinstance(self.password, six.binary_type) and u'#' in self.password:
                retval += " --password=\"%s\"" % self.password
            else:
                retval += " --password=%s" % self.password
        if self.isCrypted:
            retval += " --iscrypted"
        if self.shell:
            retval += " --shell=%s" % self.shell
        if self.uid:
            retval += " --uid=%s" % self.uid

        return retval

class F8_UserData(FC6_UserData):
    removedKeywords = FC6_UserData.removedKeywords
    removedAttrs = FC6_UserData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC6_UserData.__init__(self, *args, **kwargs)
        self.lock = kwargs.get("lock", False)

    def _getArgsAsStr(self):
        retval = FC6_UserData._getArgsAsStr(self)

        if self.lock:
            retval += " --lock"

        return retval

class F12_UserData(F8_UserData):
    removedKeywords = F8_UserData.removedKeywords
    removedAttrs = F8_UserData.removedAttrs

    def __init__(self, *args, **kwargs):
        F8_UserData.__init__(self, *args, **kwargs)
        self.gecos = kwargs.get("gecos", "")

    def _getArgsAsStr(self):
        retval = F8_UserData._getArgsAsStr(self)

        if self.gecos:
            retval += " --gecos=\"%s\"" % (self.gecos,)

        return retval

class F19_UserData(F12_UserData):
    removedKeywords = F12_UserData.removedKeywords
    removedAttrs = F12_UserData.removedAttrs

    def __init__(self, *args, **kwargs):
        F12_UserData.__init__(self, *args, **kwargs)
        self.gid = kwargs.get("gid", None)

    def _getArgsAsStr(self):
        retval = F12_UserData._getArgsAsStr(self)

        if self.gid:
            retval += " --gid=%d" % (self.gid,)

        return retval


class FC6_User(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.userList = kwargs.get("userList", [])

    def __str__(self):
        retval = ""
        for user in self.userList:
            retval += user.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="user", description="""
                            Creates a new user on the system.""",
                            version=FC6)
        op.add_argument("--groups", type=commaSplit, version=FC6, help="""
                        In addition to the default group, a comma separated
                        list of group names the user should belong to. Any groups
                        that do not already exist will be created. If the group
                        already exists with a different GID, an error will
                        be raised.""",
                        metavar="<group1>,<group2>,...,<groupN>")
        op.add_argument("--homedir", version=FC6, help="""
                        The home directory for the user. If not provided, this
                        defaults to /home/.""")
        op.add_argument("--iscrypted", dest="isCrypted", action="store_true",
                        default=False, version=FC6, help="""
                        If specified, consider the password provided by
                        ``--password`` already encrypted. This is the default.
                        """)
        op.add_argument("--name", required=True, version=FC6, help="""
                        Provides the name of the user. This option is required.
                        """)
        op.add_argument("--password", version=FC6, help="""
                        The new user's password. If not provided, the account
                        will be locked by default. If this is present, the
                        password argument is assumed to already be encrypted.
                        ``--plaintext`` has the opposite effect - the password
                        argument is assumed to not be encrypted. To create an
                        encrypted password you can use python::

                        ``python -c 'import crypt; print(crypt.crypt("My Password", "$6$My Sault"))'``

                        This will generate sha512 crypt of your password using
                        your provided salt.""")
        op.add_argument("--shell", version=FC6, help="""
                        The user's login shell. If not provided, this defaults
                        to the system default.""")
        op.add_argument("--uid", type=int, metavar="INT", version=FC6, help="""
                        The user's UID. If not provided, this defaults to the
                        next available non-system UID.""")
        return op

    def parse(self, args):
        ud = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_obj(ns, ud)
        ud.lineno = self.lineno

        # Check for duplicates in the data list.
        if ud in self.dataList():
            warnings.warn(_("A user with the name %s has already been defined.") % ud.name, KickstartParseWarning)

        return ud

    def dataList(self):
        return self.userList

    @property
    def dataClass(self):
        return self.handler.UserData

class F8_User(FC6_User):
    removedKeywords = FC6_User.removedKeywords
    removedAttrs = FC6_User.removedAttrs

    def _getParser(self):
        op = FC6_User._getParser(self)
        op.add_argument("--lock", action="store_true", default=False,
                        version=F8, help="""
                        If this is present, the new user account is locked by
                        default. That is, the user will not be able to login
                        from the console.""")
        op.add_argument("--plaintext", dest="isCrypted", version=F8,
                        action="store_false", help="""
                        If specified, consider the password provided by
                        ``--password`` to be plain text.""")
        return op

class F12_User(F8_User):
    removedKeywords = F8_User.removedKeywords
    removedAttrs = F8_User.removedAttrs

    def _getParser(self):
        op = F8_User._getParser(self)
        op.add_argument("--gecos", version=F12, help="""
                        Provides the GECOS information for the user. This is a
                        string of various system-specific fields separated by a
                        comma. It is frequently used to specify the user's full
                        name, office number, and the like. See ``man 5 passwd``
                        for more details.""")
        return op

class F19_User(F12_User):
    removedKeywords = F12_User.removedKeywords
    removedAttrs = F12_User.removedAttrs

    def _getParser(self):
        op = F12_User._getParser(self)
        op.add_argument("--gid", type=int, metavar="INT", version=F19, help="""
                        The GID of the user's primary group. If not provided,
                        this defaults to the next available non-system GID.""")
        return op

class F24_User(F19_User):
    removedKeywords = F19_User.removedKeywords
    removedAttrs = F19_User.removedAttrs

    def _getParser(self):
        op = F19_User._getParser(self)
        op.add_argument("--groups", type=commaSplit, version=F24, help="""
                        The group name can optionally be followed by a GID in
                        parenthesis, for example, ``newgroup(5002)``.""")
        return op



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest

class FC3_TestCase(CommandTest):
    command = "ignoredisk"

    def runTest(self):
        # pass
        self.assert_parse("ignoredisk --drives=sda", "ignoredisk --drives=sda\n")
        self.assert_parse("ignoredisk --drives=sda,sdb", "ignoredisk --drives=sda,sdb\n")

        # fail
        # wrong option name
        self.assert_parse_error("ignoredisk --devices=sda")
        # missing arguments
        self.assert_parse_error("ignoredisk --drives")
        # empty
        self.assert_parse_error("ignoredisk")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.ignoredisk = []
        self.assertEqual(cmd.__str__(), "")


class F8_TestCase(FC3_TestCase):
    def runTest(self):
        # Run parents class tests
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("ignoredisk --drives=sda", "ignoredisk --drives=sda\n")
        self.assert_parse("ignoredisk --drives=sda,sdb", "ignoredisk --drives=sda,sdb\n")
        self.assert_parse("ignoredisk --only-use=sda", "ignoredisk --only-use=sda\n")
        self.assert_parse("ignoredisk --only-use=sda,sdb", "ignoredisk --only-use=sda,sdb\n")

        # fail
        # missing arguments
        self.assert_parse_error("ignoredisk --only-use")
        # wrong option name
        self.assert_parse_error("ignoredisk --devices=sda")
        # missing arguments
        self.assert_parse_error("ignoredisk --drives")
        # empty
        self.assert_parse_error("ignoredisk")
        # both options provided
        self.assert_parse_error("ignoredisk --drives=sda --only-use=sdb")
        self.assert_parse_error("ignoredisk --only-use=sda --drives=sdb")

class RHEL6_TestCase(F8_TestCase):
    def runTest(self):
        # Run parents class tests
        F8_TestCase.runTest(self)

        # pass
        self.assert_parse("ignoredisk --interactive", "ignoredisk --interactive\n")

        # fail
        # both options provided
        self.assert_parse_error("ignoredisk --drives=sda --interactive")
        self.assert_parse_error("ignoredisk --interactive --drives=sda")
        self.assert_parse_error("ignoredisk --only-use=sda --interactive")
        self.assert_parse_error("ignoredisk --interactive --only-use=sda")
        self.assert_parse_error("ignoredisk --interactive --drives=sda --only-use=sdb")
        self.assert_parse_error("ignoredisk --only-use=sda --drives=sdb --interactive")

class F29_TestCase(F8_TestCase):
    def runTest(self):
        F8_TestCase.runTest(self)
        self.assert_deprecated("ignoredisk", "--interactive")

class RHEL8_TestCase(F29_TestCase):
    pass

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3
from pykickstart.base import KickstartCommand
from pykickstart.constants import FIRSTBOOT_DEFAULT, FIRSTBOOT_SKIP, FIRSTBOOT_RECONFIG
from pykickstart.options import KSOptionParser

class FC3_Firstboot(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.firstboot = kwargs.get("firstboot", None)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.firstboot is None:
            return retval

        if self.firstboot == FIRSTBOOT_SKIP:
            retval += "firstboot --disable\n"
        elif self.firstboot == FIRSTBOOT_DEFAULT:
            retval += "# Run the Setup Agent on first boot\nfirstboot --enable\n"
        elif self.firstboot == FIRSTBOOT_RECONFIG:
            retval += "# Run the Setup Agent on first boot\nfirstboot --reconfig\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="firstboot", description="""
                            Determine whether the Setup Agent starts the first
                            time the system is booted. If enabled, the
                            ``initial-setup`` package must be installed. If not
                            specified, the setup agent (initial-setup) is disabled
                            by default.""",
                            version=FC3)
        op.add_argument("--disable", "--disabled", dest="firstboot",
                        action="store_const", const=FIRSTBOOT_SKIP, version=FC3,
                        help="""The Setup Agent is not started the first time the
                        system boots.""")
        op.add_argument("--enable", "--enabled", dest="firstboot",
                        action="store_const", const=FIRSTBOOT_DEFAULT, version=FC3,
                        help="""The Setup Agent is started the first time the
                        system boots.""")
        op.add_argument("--reconfig", dest="firstboot", version=FC3,
                        action="store_const", const=FIRSTBOOT_RECONFIG,
                        help="""
                        Enable the Setup Agent to start at boot time in
                        reconfiguration mode. This mode enables the language,
                        mouse, keyboard, root password, security level,
                        time zone, and networking configuration options in
                        addition to the default ones.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.firstboot = ns.firstboot
        return self



import unittest
from tests.baseclass import CommandTest


class F26_TestCase(CommandTest):
    command = "snapshot"

    def runTest(self):
        # pass
        self.assert_parse("snapshot vg00/lv --name test --when post-install",
                          "snapshot vg00/lv --name=test --when=post-install\n")
        self.assert_parse("snapshot vg00/lv --name=test --when=post-install",
                          "snapshot vg00/lv --name=test --when=post-install\n")
        self.assert_parse("snapshot vg00/lv --name=test --when pre-install",
                          "snapshot vg00/lv --name=test --when=pre-install\n")
        self.assert_parse("snapshot vg00/lv --name=test --when=pre-install",
                          "snapshot vg00/lv --name=test --when=pre-install\n")

        # missing required field
        self.assert_parse_error("snapshot")
        self.assert_parse_error("snapshot vg00/lv")
        self.assert_parse_error("snapshot --name=test")
        self.assert_parse_error("snapshot vg00/lv --name=test")
        self.assert_parse_error("snapshot vg00/lv --when=post-install")
        self.assert_parse_error("snapshot --name")
        self.assert_parse_error("snapshot --when")
        self.assert_parse_error("snapshot --name=test --when")

        # bad lv name
        self.assert_parse_error("snapshot lv --name test --when post-install")
        self.assert_parse_error("snapshot vg_lv --name test --when post-install")
        self.assert_parse_error("snapshot vg//lv --name test --when post-install")
        self.assert_parse_error("snapshot vg\\lv --name test --when post-install")
        self.assert_parse_error("snapshot vg=lv --name test --when post-install")

        # bad when option
        self.assert_parse_error("snapshot lv --name test --when error")
        self.assert_parse_error("snapshot lv --name test --when=error")
        self.assert_parse_error("snapshot lv --name test --when post-install --when pre-install")
        self.assert_parse_error("snapshot lv --name test --when=post-install --when pre-install")
        self.assert_parse_error("snapshot lv --name test --when=post-install --when=pre-install")

        # nonsensical parameter test
        self.assert_parse_error("snapshot --nonsense")
        self.assert_parse_error("snapshot nonsense")


if __name__ == "__main__":
    unittest.main()



#
# Paul W. Frields <pfrields@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest

class FC3_TestCase(CommandTest):
    command = "driverdisk"

    def runTest(self):
        # pass
        self.assert_parse("driverdisk /dev/sdb2", "driverdisk /dev/sdb2\n")
        self.assert_parse("driverdisk --source=http://10.0.0.1/disk.img", "driverdisk --source=http://10.0.0.1/disk.img\n")

        if "--type" in self.optionList:
            self.assert_parse("driverdisk /dev/sdb2 --type=vfat", "driverdisk /dev/sdb2 --type=vfat\n")
            # pass - need separate tests per fstype?
            self.assert_parse("driverdisk /dev/sdb2 --type=ext2", "driverdisk /dev/sdb2 --type=ext2\n")
        else:
            self.assert_parse("driverdisk /dev/sdb2", "driverdisk /dev/sdb2\n")
            # pass - need separate tests per fstype?
            self.assert_parse("driverdisk /dev/sdb2", "driverdisk /dev/sdb2\n")

        # fail - no arguments
        self.assert_parse_error("driverdisk")
        # fail - unrecognized argument
        self.assert_parse_error("driverdisk --bogus-option")
        # fail - spurious argument or extra partition
        self.assert_parse_error("driverdisk /dev/sdb2 foobar")
        # fail - specifying both partition and source
        self.assert_parse_error("driverdisk /dev/sdb2 --source=http://10.0.0.1/disk.img")

        # extra test coverage
        ddd = self.handler().DriverDiskData()
        ddd.source = None
        self.assertEqual(ddd._getArgsAsStr(), "")

        cmd = self.handler().commands[self.command]
        self.assertEqual(cmd.__str__(), "")
        cmd.driverdiskList = [ddd]
        self.assertEqual(cmd.__str__(), "driverdisk \n")
        self.assertEqual(cmd.dataList(), [ddd])

class FC4_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("driverdisk --biospart=0x82", "driverdisk --biospart=0x82\n")
        self.assert_parse("driverdisk --biospart=0x80p1", "driverdisk --biospart=0x80p1\n")

        # fail - no arguments
        self.assert_parse_error("driverdisk --biospart")
        # fail - specifying both biospart and partition
        self.assert_parse_error("driverdisk /dev/sdb2 --biospart=0x82")
        # fail - specifying both biospart and source
        self.assert_parse_error("driverdisk --source=http://10.0.0.1/disk.img --biospart=0x82")

class F12_TestCase(FC4_TestCase):
    def runTest(self):
        FC4_TestCase.runTest(self)
        self.assert_deprecated("driverdisk", "--type=ext4")

class F14_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)
        self.assert_removed("driverdisk", "--type=ext4")

if __name__ == "__main__":
    unittest.main()



#
# Brian C. Lane <bcl@redhat.com>
#
# Copyright 2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F22
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser
import warnings

from pykickstart.i18n import _

class F22_SshKeyData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.username = kwargs.get("username", None)
        self.key = kwargs.get("key", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.username == y.username

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)

        retval += "sshkey"
        retval += self._getArgsAsStr() + '\n'

        return retval

    def _getArgsAsStr(self):
        retval = ""

        retval += " --username=%s" % self.username
        retval += ' "%s"' % self.key
        return retval

class F22_SshKey(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.sshUserList = kwargs.get("sshUserList", [])

    def __str__(self):
        retval = ""
        for user in self.sshUserList:
            retval += user.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="sshkey", description="""
                            This installs a ssh key to the authorized_keys file
                            of the specified user on the installed system.""",
                            epilog="""
                            Note that the key should be quoted, if it contains
                            spaces and the user should exist (or be root)
                            either via creation by a package install or the
                            kickstart ``user`` command.""", version=F22)
        op.add_argument("--username", required=True, metavar="<user>",
                        version=F22, help="""
                        User for which to install the specified key.""")
        op.add_argument('sshkey', metavar='"ssh key"', nargs=1,
                        version=F22, help="""
                        The content of the ssh key to install.""")
        return op

    def parse(self, args):
        ud = self.dataClass()   # pylint: disable=not-callable
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        assert len(ns.sshkey) == 1

        if extra:
            mapping = {"command": "sshkey", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_obj(ns, ud)
        ud.key = ns.sshkey[0]
        ud.lineno = self.lineno

        if ud in self.dataList():
            warnings.warn(_("An ssh user with the name %s has already been defined.") % ud.username, KickstartParseWarning)

        return ud

    def dataList(self):
        return self.sshUserList

    @property
    def dataClass(self):
        return self.handler.SshKeyData



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest, CommandSequenceTest

from pykickstart.errors import KickstartError, KickstartParseWarning
from pykickstart.version import FC6

class FC6_TestCase(CommandTest):
    command = "repo"

    def runTest(self, urlRequired=True):
        # pass
        self.assert_parse("repo --name=blah --baseurl=http://www.domain.com",
                          "repo --name=\"blah\" --baseurl=http://www.domain.com\n")
        self.assert_parse("repo --name=blah --mirrorlist=http://www.domain.com",
                          "repo --name=\"blah\" --mirrorlist=http://www.domain.com\n")

        # equality
        self.assertEqual(self.assert_parse("repo --name=left --baseurl=http://wherever"), self.assert_parse("repo --name=left --baseurl=http://wherever"))
        self.assertEqual(self.assert_parse("repo --name=left --baseurl=http://wherever"), self.assert_parse("repo --name=left --baseurl=http://somewhere"))
        self.assertNotEqual(self.assert_parse("repo --name=left --baseurl=http://wherever"), None)
        self.assertNotEqual(self.assert_parse("repo --name=left --baseurl=http://wherever"), self.assert_parse("repo --name=right --baseurl=http://wherever"))

        # fail
        # missing required option --name
        self.assert_parse_error("repo --baseurl=www.domain.com")
        self.assert_parse_error("repo --name --baseurl=www.domain.com")
        # missing one of required options --baseurl or --mirrorlist
        if urlRequired:
            self.assert_parse_error("repo --name=blah")
            self.assert_parse_error("repo --name=blah --baseurl")
            self.assert_parse_error("repo --name=blah --mirrorlist")
        # only one of --baseurl or --mirrorlist must be specified
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com --mirrorlist=www.domain.com")
        # unknown option
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com --unknown")
        # not expected argument
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com blah")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.repoList = "--name=blah"
        self.assertEqual(cmd.__str__(), "--name=blah")

        data = self.handler().RepoData()
        data.baseurl = ""
        data.mirrorlist = ""
        self.assertEqual(data._getArgsAsStr(), "")

class FC6_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC6

    def runTest(self):
        self.assert_parse("""
repo --name=repoA --baseurl=http://www.domain.com
repo --name=repoB --baseurl=http://www.domain.com""")

        self.assert_parse_error("""
repo --name=repoA --baseurl=http://www.domain.com
repo --name=repoA --baseurl=http://www.domain.com""", KickstartParseWarning)

class F8_TestCase(FC6_TestCase):
    def runTest(self, urlRequired=True):
        # run FC6 test case
        FC6_TestCase.runTest(self, urlRequired=urlRequired)

        # pass
        self.assert_parse("repo --name=blah --baseurl=www.domain.com --cost=10 --excludepkgs=pkg1,pkg2 --includepkgs=pkg3,pkg4",
                          "repo --name=\"blah\" --baseurl=www.domain.com --cost=10 --includepkgs=\"pkg3,pkg4\" --excludepkgs=\"pkg1,pkg2\"\n")
        self.assert_parse("repo --name=blah --baseurl=123xyz --cost=10 --excludepkgs=pkg1,pkg2 --includepkgs=pkg3,pkg4",
                          "repo --name=\"blah\" --baseurl=123xyz --cost=10 --includepkgs=\"pkg3,pkg4\" --excludepkgs=\"pkg1,pkg2\"\n")

        # fail
        # missing required arguments
        for opt in ("--cost", "--includepkgs", "--excludepkgs"):
            self.assert_parse_error("repo --name=blah --baseurl=www.domain.com %s" % opt)
        # --cost argument not integer
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com --cost=high")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.handler.method.url = "http://example.com"
        rd = cmd.methodToRepo()
        self.assertEqual(rd.name, "ks-method-url")
        self.assertEqual(rd.baseurl, "http://example.com")

        cmd.handler.method.url = ""
        with self.assertRaises(KickstartError):
            cmd.methodToRepo()

class F11_TestCase(F8_TestCase):
    def runTest(self, urlRequired=True):
        # run F8 test case
        F8_TestCase.runTest(self, urlRequired=urlRequired)

        # pass
        for val in ("1", "true", "on"):
            self.assert_parse("repo --name=blah --baseurl=www.domain.com --cost=10 --excludepkgs=pkg1,pkg2 --includepkgs=pkg3,pkg4 --ignoregroups=%s" % val,
                              "repo --name=\"blah\" --baseurl=www.domain.com --cost=10 --includepkgs=\"pkg3,pkg4\" --excludepkgs=\"pkg1,pkg2\" --ignoregroups=true\n")
        for val in ("0", "false", "off"):
            self.assert_parse("repo --name=blah --baseurl=www.domain.com --cost=10 --excludepkgs=pkg1,pkg2 --includepkgs=pkg3,pkg4 --ignoregroups=%s" % val,
                              "repo --name=\"blah\" --baseurl=www.domain.com --cost=10 --includepkgs=\"pkg3,pkg4\" --excludepkgs=\"pkg1,pkg2\"\n")

        # fail
        # missing --ignoregroups argument
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com --ignoregroups")

class F13_TestCase(F11_TestCase):
    def runTest(self, urlRequired=True):
        # run F11 test case
        F11_TestCase.runTest(self, urlRequired=urlRequired)

        # pass
        self.assert_parse("repo --name=blah --baseurl=www.domain.com --proxy=http://someplace/wherever",
                          "repo --name=\"blah\" --baseurl=www.domain.com --proxy=\"http://someplace/wherever\"\n")
        self.assert_parse("repo --name=blah --baseurl=www.domain.com --proxy=\"http://someplace/wherever\"",
                          "repo --name=\"blah\" --baseurl=www.domain.com --proxy=\"http://someplace/wherever\"\n")

        # fail
        # missing --proxy argument
        self.assert_parse_error("repo --name=blah --baseurl=www.domain.com --proxy")

class F14_TestCase(F13_TestCase):
    def runTest(self, urlRequired=True):
        F13_TestCase.runTest(self, urlRequired=urlRequired)
        #pass
        self.assert_parse("repo --name=blah --baseurl=https://www.domain.com --noverifyssl",
                          "repo --name=\"blah\" --baseurl=https://www.domain.com --noverifyssl\n")
        #fail
        self.assert_parse_error("repo --name=blah --baseurl=https://www.domain.com --noverifyssl=yeeeaah")

class F15_TestCase(F14_TestCase):
    def runTest(self, urlRequired=False):
        F14_TestCase.runTest(self, urlRequired=urlRequired)

class F21_TestCase(F15_TestCase):
    def runTest(self, urlRequired=False):
        F15_TestCase.runTest(self, urlRequired=urlRequired)
        #pass
        self.assert_parse("repo --name=blah --baseurl=https://www.domain.com --install",
                          "repo --name=\"blah\" --baseurl=https://www.domain.com --install\n")
        #fail
        self.assert_parse_error("repo --name=blah --baseurl=https://www.domain.com --install=yeeeaah")

        # only one of --baseurl --mirrorlist may be specified
        self.assert_parse_error("repo --name=blah --baseurl=https://www.domain.com --mirrorlist=https://www.domain.com/mirror")

class F27_TestCase(F21_TestCase):
    def runTest(self, urlRequired=False):
        F21_TestCase.runTest(self, urlRequired=urlRequired)

        self.assert_parse("repo --name=blah --metalink=https://www.domain.com/metalink")

        # only one of --baseurl --mirrorlist --metalink may be specified
        self.assert_parse_error("repo --name=blah --metalink=https://www.domain.com/metalink --mirrorlist=https://www.domain.com/mirror")
        self.assert_parse_error("repo --name=blah --baseurl=https://www.domain.com --metalink=https://www.domain.com/metalink")

class RHEL8_TestCase(F27_TestCase):
    def runTest(self, urlRequired=False):
        F27_TestCase.runTest(self, urlRequired=urlRequired)

        # pass
        self.assert_parse("repo --name=test --baseurl=https://www.domain.com --sslclientcert=file:///foo/bar",
                          "repo --name=\"test\" --baseurl=https://www.domain.com --sslclientcert=\"file:///foo/bar\"\n")
        self.assert_parse("repo --name=test --baseurl=https://www.domain.com --sslclientkey=file:///foo/bar",
                          "repo --name=\"test\" --baseurl=https://www.domain.com --sslclientkey=\"file:///foo/bar\"\n")
        self.assert_parse("repo --name=test --baseurl=https://www.domain.com --sslcacert=file:///foo/bar",
                          "repo --name=\"test\" --baseurl=https://www.domain.com --sslcacert=\"file:///foo/bar\"\n")

        # fail: all of these take arguments
        self.assert_parse_error("repo --name=test --sslclientcert")
        self.assert_parse_error("repo --name=test --sslclientkey")
        self.assert_parse_error("repo --name=test --sslcacert")

if __name__ == "__main__":
    unittest.main()



import unittest
import os
import tempfile
import six

from pykickstart import load
from pykickstart.errors import KickstartError
from signal import SIGTERM

class LoadTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self._content = """
auth --enableshadow --passalgo=sha512
graphical
firstboot --enable
ignoredisk --only-use=vda
keyboard --vckeymap=cz --xlayouts='cz'
lang cs_CZ.UTF-8

network  --bootproto=dhcp --device=ens3 --ipv6=auto --activate
network  --hostname=test
rootpw password
timezone Europe/Prague --isUtc
user --groups=wheel --name=testuser --password=password --iscrypted --gecos="Test User"
xconfig  --startxonboot
bootloader --location=mbr --boot-drive=vda
autopart --type=lvm
clearpart --all --initlabel --drives=vda

%packages
@^xfce-desktop-environment
@xfce-apps
@xfce-media
%end

%addon com_redhat_kdump --disable --reserve-mb='128'
%end
"""

    def setUp(self):
        (handle, self._path) = tempfile.mkstemp(prefix="testfile-", text=True)
        os.write(handle, self._content.encode("utf-8"))
        os.close(handle)

    def tearDown(self):
        os.unlink(self._path)

class Load_To_String_TestCase(LoadTest):
    def runTest(self):
        self.assertEqual(self._content, load.load_to_str(self._path))

class Load_To_File_TestCase(LoadTest):
    def __init__(self, *args, **kwargs):
        LoadTest.__init__(self, *args, **kwargs)
        self._target_path = ""

    def runTest(self):
        (handle, self._target_path) = tempfile.mkstemp(prefix="testfile", text=True)
        os.close(handle)
        target_path = load.load_to_file(self._path, self._target_path)
        self.assertEqual(target_path, self._target_path)
        with open(self._target_path, 'r') as f:
            self.assertEqual(self._content, f.read())
        self.assertRaises(KickstartError, load.load_to_file, "/tmp/foo", "/tmp/bar")

    def tearDown(self):
        super(Load_To_File_TestCase, self).tearDown()
        os.unlink(self._target_path)

class Load_From_URL_Test(LoadTest):
    def setUp(self):
        super(Load_From_URL_Test, self).setUp()

        # Disable logging in the handler, mostly to keep the HTTPS binary garbage off the screen
        httphandler = six.moves.SimpleHTTPServer.SimpleHTTPRequestHandler

        def shutup(*args, **kwargs):
            pass
        httphandler.log_message = shutup

        self._server = six.moves.BaseHTTPServer.HTTPServer(('127.0.0.1', 0), httphandler)
        httpd_port = self._server.server_port
        self._httpd_pid = os.fork()
        if self._httpd_pid == 0:
            os.chdir(os.path.dirname(self._path))
            self._server.serve_forever()
        self._url = 'http://127.0.0.1:%d/%s' % (httpd_port, os.path.basename(self._path))
        # wrong URL (HTTPS request won't be handled correctly by the HTTP server)
        self._url_https = "https" + self._url.lstrip("http")

    def tearDown(self):
        super(Load_From_URL_Test, self).tearDown()
        self._server.server_close()
        os.kill(self._httpd_pid, SIGTERM)

class Load_From_URL_To_Str_TestCase(Load_From_URL_Test):
    def runTest(self):
        self.assertEqual(self._content, load.load_to_str(self._url))
        self.assertRaises(KickstartError, load.load_to_str, self._url_https)

class Load_From_URL_To_File_TestCase(Load_From_URL_Test):
    def setUp(self):
        super(Load_From_URL_To_File_TestCase, self).setUp()
        (handle, self._target_path) = tempfile.mkstemp(prefix="testfile", text=True)
        os.close(handle)

    def runTest(self):
        target_path = load.load_to_file(self._url, self._target_path)
        self.assertEqual(target_path, self._target_path)
        with open(self._target_path, 'r') as f:
            self.assertEqual(self._content, f.read())
        self.assertEqual(self._content, load.load_to_str(self._url))
        self.assertRaises(KickstartError,
                          load.load_to_file,
                          self._url_https,
                          self._target_path)

    def tearDown(self):
        super(Load_From_URL_To_File_TestCase, self).tearDown()
        os.unlink(self._target_path)

if __name__ == "__main__":
    unittest.main()



#
# sections.py:  Kickstart file sections.
#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2011-2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
This module exports the classes that define a section of a kickstart file.  A
section is a chunk of the file starting with a %tag and ending with a %end.
Examples of sections include %packages, %pre, and %post.

You may use this module to define your own custom sections which will be
treated just the same as a predefined one by the kickstart parser.  All that
is necessary is to create a new subclass of Section and call
parser.registerSection with an instance of your new class.
"""
from pykickstart.constants import KS_SCRIPT_PRE, KS_SCRIPT_POST, KS_SCRIPT_TRACEBACK, \
                                  KS_SCRIPT_PREINSTALL, KS_SCRIPT_ONERROR, \
                                  KS_MISSING_IGNORE, KS_MISSING_PROMPT
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import FC4, F7, F9, F18, F21, F22, F24, RHEL7
from pykickstart.i18n import _

class Section(object):
    """The base class for defining kickstart sections.  You are free to
       subclass this as appropriate.

       Class attributes:

       allLines    -- Does this section require the parser to call handleLine
                      for every line in the section, even blanks and comments?
       sectionOpen -- The string that denotes the start of this section.  You
                      must start your tag with a percent sign.
       timesSeen   -- This attribute is for informational purposes only.  It is
                      incremented every time handleHeader is called to keep
                      track of the number of times a section of this type is
                      seen.
    """
    allLines = False
    sectionOpen = ""
    timesSeen = 0

    def __init__(self, handler, **kwargs):
        """Create a new Script instance.  At the least, you must pass in an
           instance of a baseHandler subclass.

           Valid kwargs:

           dataObj -- A class that should be populated by this Section.  It almost
                      always should be Script, or some subclass of it.
        """
        self.handler = handler
        self.version = self.handler.version

        self.dataObj = kwargs.get("dataObj", None)

    def finalize(self):
        """This method is called when the %end tag for a section is seen.  It
           is not required to be provided.
        """

    def handleLine(self, line):
        """This method is called for every line of a section.  Take whatever
           action is appropriate.  While this method is not required to be
           provided, not providing it does not make a whole lot of sense.

           Arguments:

           line -- The complete line, with any trailing newline.
        """

    # pylint: disable=unused-argument
    def handleHeader(self, lineno, args):
        """This method is called when the opening tag for a section is seen.
           Not all sections will need this method, though all provided with
           kickstart include one.

           Arguments:

           args -- A list of all strings passed as arguments to the section
                   opening tag.
        """
        self.timesSeen += 1
    # pylint: enable=unused-argument

    @property
    def seen(self):
        """This property is given for consistency with KickstartCommand objects
           only.  It simply returns whether timesSeen is non-zero.
        """
        return self.timesSeen > 0

class NullSection(Section):
    """This defines a section that pykickstart will recognize but do nothing
       with.  If the parser runs across a %section that has no object registered,
       it will raise an error.  Sometimes, you may want to simply ignore those
       sections instead.  This class is useful for that purpose.
    """
    allLines = True

    def __init__(self, *args, **kwargs):
        """Create a new NullSection instance.  You must pass a sectionOpen
           parameter (including a leading '%') for the section you wish to
           ignore.
        """
        Section.__init__(self, *args, **kwargs)
        self.sectionOpen = kwargs.get("sectionOpen")
        self._args = []
        self._body = []

    def handleHeader(self, lineno, args):
        self._args = args

    def handleLine(self, line):
        self._body.append(line)

    def finalize(self):
        body = "\n".join(self._body)
        if body:
            s = "%s\n%s\n%%end" % (" ".join(self._args), body)
        else:
            s = "%s\n%%end" % " ".join(self._args)

        self.handler._null_section_strings.append(s)

        self._args = []
        self._body = []

class ScriptSection(Section):
    allLines = True
    _description = ""
    _epilog = ""
    _title = ""

    def __init__(self, *args, **kwargs):
        Section.__init__(self, *args, **kwargs)
        self._script = {}
        self._resetScript()

    def _getParser(self):
        op = KSOptionParser(prog=self.sectionOpen,
                            description=self._description,
                            epilog=self._epilog,
                            version=self.version,
                            addVersion=False)
        op.add_argument("--erroronfail", dest="errorOnFail", action="store_true",
                        default=False, help="""
                        If the error script fails, this option will cause an
                        error dialog to be displayed and will halt installation.
                        The error message will direct you to where the cause of
                        the failure is logged.""", introduced=FC4)
        op.add_argument("--interpreter", dest="interpreter", default="/bin/sh",
                        introduced=FC4, metavar="/usr/bin/python", help="""
                        Allows you to specify a different scripting language,
                        such as Python. Replace /usr/bin/python with the
                        scripting language of your choice.
                        """)
        op.add_argument("--log", "--logfile", dest="log", introduced=FC4,
                        help="""
                        Log all messages from the script to the given log file.
                        """)
        return op

    def _resetScript(self):
        self._script = {"interp": "/bin/sh", "log": None, "errorOnFail": False,
                        "lineno": None, "chroot": False, "body": []}

    def handleLine(self, line):
        self._script["body"].append(line)

    def finalize(self):
        if " ".join(self._script["body"]).strip() == "":
            return

        kwargs = {"interp": self._script["interp"],
                  "inChroot": self._script["chroot"],
                  "lineno": self._script["lineno"],
                  "logfile": self._script["log"],
                  "errorOnFail": self._script["errorOnFail"],
                  "type": self._script["type"]}

        if self.dataObj is not None:
            s = self.dataObj(self._script["body"], **kwargs)
            self._resetScript()
            self.handler.scripts.append(s)

    def handleHeader(self, lineno, args):
        """Process the arguments to a %pre/%post/%traceback header for later
           setting on a Script instance once the end of the script is found.
           This method may be overridden in a subclass if necessary.
        """
        Section.handleHeader(self, lineno, args)
        op = self._getParser()

        ns = op.parse_args(args=args[1:], lineno=lineno)

        self._script["interp"] = ns.interpreter
        self._script["lineno"] = lineno
        self._script["log"] = ns.log
        self._script["errorOnFail"] = ns.errorOnFail
        if hasattr(ns, "nochroot"):
            self._script["chroot"] = not ns.nochroot

class PreScriptSection(ScriptSection):
    sectionOpen = "%pre"
    _title = "Pre-installation script"
    _description = """
        You can add commands to run on the system immediately after the ks.cfg
        has been parsed and the lang, keyboard, and url options have been
        processed. This section must be at the end of the kickstart file (after
        the commands) and must start with the %pre command. You can access the
        network in the %pre section; however, name service has not been
        configured at this point, so only IP addresses will work.

        Preinstallation scripts are required to be closed with %end.

        If your script spawns a daemon process, you must make sure to close
        ``stdout`` and ``stderr``. Doing so is standard procedure for creating
        daemons. If you do not close these file descriptors, the installation
        will appear hung as anaconda waits for an EOF from the script.

        .. note::

            The pre-install script is not run in the chroot environment.
    """
    _epilog = """
    Example
    -------

    Here is an example %pre section::

        %pre
        #!/bin/bash
        hds=""
        mymedia=""

        for file in /sys/block/sd*; do
        hds="$hds $(basename $file)"
        done

        set $hds
        numhd=$(echo $#)

        drive1=$(echo $hds | cut -d' ' -f1)
        drive2=$(echo $hds | cut -d' ' -f2)


        if [ $numhd == "2" ]  ; then
            echo "#partitioning scheme generated in %pre for 2 drives" > /tmp/part-include
            echo "clearpart --all" >> /tmp/part-include
            echo "part /boot --fstype ext4 --size 512 --ondisk sda" >> /tmp/part-include
            echo "part / --fstype ext4 --size 10000 --grow --ondisk sda" >> /tmp/part-include
            echo "part swap --recommended --ondisk $drive1" >> /tmp/part-include
            echo "part /home --fstype ext4 --size 10000 --grow --ondisk sdb" >> /tmp/part-include
        else
            echo "#partitioning scheme generated in %pre for 1 drive" > /tmp/part-include
            echo "clearpart --all" >> /tmp/part-include
            echo "part /boot --fstype ext4 --size 521" >> /tmp/part-include
            echo "part swap --recommended" >> /tmp/part-include
            echo "part / --fstype ext4 --size 2048" >> /tmp/part-include
            echo "part /home --fstype ext4 --size 2048 --grow" >> /tmp/part-include
        fi
        %end

    This script determines the number of hard drives in the system and
    writes a text file with a different partitioning scheme depending on
    whether it has one or two drives. Instead of having a set of
    partitioning commands in the kickstart file, include the line:

    ``%include /tmp/part-include``

    The partitioning commands selected in the script will be used.
    """

    def _resetScript(self):
        ScriptSection._resetScript(self)
        self._script["type"] = KS_SCRIPT_PRE

class PreInstallScriptSection(ScriptSection):
    sectionOpen = "%pre-install"
    _title = "Pre-install Script"
    _description="""
    You can use the %pre-install section to run commands after the system has been
    partitioned, filesystems created, and everything is mounted under /mnt/sysimage
    Like %pre these scripts do not run in the chrooted environment.

    Each %pre-install section is required to be closed with a corresponding %end.
    """

    def _resetScript(self):
        ScriptSection._resetScript(self)
        self._script["type"] = KS_SCRIPT_PREINSTALL

class PostScriptSection(ScriptSection):
    sectionOpen = "%post"
    _title = "Post-installation Script"
    _description="""
    You have the option of adding commands to run on the system once the
    installation is complete. This section must be at the end of the
    kickstart file and must start with the %post command. This section is
    useful for functions such as installing additional software and
    configuring an additional nameserver.

    You may have more than one %post section, which can be useful for cases
    where some post-installation scripts need to be run in the chroot and
    others that need access outside the chroot.

    Each %post section is required to be closed with a corresponding %end.

    If you configured the network with static IP information, including a
    nameserver, you can access the network and resolve IP addresses in the %post
    section.  If you configured the network for DHCP, the /etc/resolv.conf file
    has not been completed when the installation executes the %post section. You
    can access the network, but you can not resolve IP addresses. Thus, if you
    are using DHCP, you must specify IP addresses in the %post section.

    If your script spawns a daemon process, you must make sure to close stdout
    and stderr.  Doing so is standard procedure for creating daemons.  If you do
    not close these file descriptors, the installation will appear hung as
    anaconda waits for an EOF from the script.

    The post-install script is run in a chroot environment; therefore, performing
    tasks such as copying scripts or RPMs from the installation media will not
    work.
    """

    _epilog="""
    Examples
    --------

    Run a script named ``runme`` from an NFS share::

        %post
        mkdir /mnt/temp
        mount 10.10.0.2:/usr/new-machines /mnt/temp
        open -s -w -- /mnt/temp/runme
        umount /mnt/temp
        %end

    Copy the file /etc/resolv.conf to the file system that was just
    installed::

        %post --nochroot
        cp /etc/resolv.conf /mnt/sysimage/etc/resolv.conf
        %end

    **If your kickstart is being interpreted by the livecd-creator tool, you should
    replace /mnt/sysimage above with $INSTALL_ROOT.**
    """

    def _getParser(self):
        op = ScriptSection._getParser(self)
        op.add_argument("--nochroot", dest="nochroot", action="store_true",
                        default=False, introduced=FC4, help="""
                        Allows you to specify commands that you would like to
                        run outside of the chroot environment.""")
        return op

    def _resetScript(self):
        ScriptSection._resetScript(self)
        self._script["chroot"] = True
        self._script["type"] = KS_SCRIPT_POST

class OnErrorScriptSection(ScriptSection):
    sectionOpen = "%onerror"
    _title = "Handling Errors"
    _description="""
    These scripts run when the installer hits a fatal error, but not necessarily
    a bug in the installer.  Some examples of these situations include errors in
    packages that have been requested to be installed, failures when starting VNC
    when requested, and error when scanning storage.  When these situations happen,
    installaton cannot continue.  The installer will run all %onerror scripts in
    the order they are provided in the kickstart file.

    In addition, %onerror scripts will be run on a traceback as well.  To be exact,
    all %onerror scripts will be run and then all %traceback scripts will be run
    afterwards.

    Each %onerror script is required to be closed with a corresponding %end.

    .. note::

        These scripts could potentially run at
        any stage in installation - early on, between making filesystems and installing
        packages, before the bootloader is installed, when attempting to reboot, and
        so on.  For this reason, these scripts cannot be run in the chroot environment
        and you should not trust anything in the installed system.  These scripts are
        primarily for testing and error reporting purposes.
    """

    def _resetScript(self):
        ScriptSection._resetScript(self)
        self._script["type"] = KS_SCRIPT_ONERROR

class TracebackScriptSection(OnErrorScriptSection):
    sectionOpen = "%traceback"
    _title = "Handling Tracebacks"
    _description="""
    These scripts run when the installer hits an internal error (a traceback, as
    they are called in Python) and cannot continue.  When this situation happens,
    the installer will display an error dialog to the screen that prompts the user
    to file a bug or reboot.  At the same time, it will run all %traceback scripts
    in the order they are provided in the kickstart file.

    Each %traceback script is required to be closed with a corresponding %end.

    .. note::

        These scripts could potentially run at
        any stage in installation - early on, between making filesystems and installing
        packages, before the bootloader is installed, when attempting to reboot, and
        so on.  For this reason, these scripts cannot be run in the chroot environment
        and you should not trust anything in the installed system.  These scripts are
        primarily for testing and error reporting purposes.
    """

    def _resetScript(self):
        OnErrorScriptSection._resetScript(self)
        self._script["type"] = KS_SCRIPT_TRACEBACK

class PackageSection(Section):
    sectionOpen = "%packages"
    _title = "Package Selection"

    def handleLine(self, line):
        h = line.partition('#')[0]
        line = h.rstrip()
        self.handler.packages.add([line])

    def _getParser(self):
        op = KSOptionParser(prog=self.sectionOpen, description="""
                            Use the %packages command to begin a kickstart file
                            section that lists the packages you would like to
                            install.

                            Packages can be specified by group or by individual
                            package name. The installation program defines
                            several groups that contain related packages. Refer
                            to the repodata/\\*comps\\*.xml file on the first CD-ROM
                            for a list of groups. Each group has an id, user
                            visibility value, name, description, and package
                            list. In the package list, the packages marked as
                            mandatory are always installed if the group is
                            selected, the packages marked default are selected
                            by default if the group is selected, and the packages
                            marked optional must be specifically selected even
                            if the group is selected to be installed.

                            In most cases, it is only necessary to list the
                            desired groups and not individual packages. Note
                            that the Core group is always selected by default,
                            so it is not necessary to specify it in the
                            %packages section.

                            The %packages section is required to be closed with
                            %end. Also, multiple %packages sections may be given.
                            This may be handy if the kickstart file is used as a
                            template and pulls in various other files with the
                            %include mechanism.

                            Here is an example %packages selection::

                                %packages
                                @X Window System
                                @GNOME Desktop Environment
                                @Graphical Internet
                                @Sound and Video
                                dhcp
                                %end

                            As you can see, groups are specified, one to a line,
                            starting with an ``@`` symbol followed by the full
                            group name as given in the comps.xml file. Groups
                            can also be specified using the id for the group,
                            such as gnome-desktop. Specify individual packages
                            with no additional characters (the dhcp line in the
                            example above is an individual package).

                            You can also specify environments using the ``@^``
                            prefix followed by full environment name as given in
                            the comps.xml file.  If multiple environments are
                            specified, only the last one specified will be used.
                            Environments can be mixed with both group
                            specifications (even if the given group is not part
                            of the specified environment) and package
                            specifications.

                            Here is an example of requesting the GNOME Desktop
                            environment to be selected for installation::

                                %packages
                                @^gnome-desktop-environment
                                %end

                            Additionally, individual packages may be specified
                            using globs. For instance::

                                %packages
                                vim*
                                kde-i18n-*
                                %end

                            This would install all packages whose names start
                            with "vim" or "kde-i18n-".

                            You can also specify which packages or groups not to
                            install from the default package list::

                                %packages
                                -autofs
                                -@Sound and Video
                                %end
                            """, epilog="""
                            Group-level options
                            -------------------

                            In addition, group lines in the %packages section
                            can take the following options:

                            ``--nodefaults``

                                Only install the group's mandatory packages, not
                                the default selections.

                            ``--optional``

                                In addition to the mandatory and default packages,
                                also install the optional packages. This means all
                                packages in the group will be installed.
                            """, version=self.version, addVersion=False)
        op.add_argument("--excludedocs", action="store_true", default=False,
                        help="""
                        Do not install any of the documentation from any packages.
                        For the most part, this means files in /usr/share/doc*
                        will not get installed though it could mean other files
                        as well, depending on how the package was built.""",
                        introduced=FC4)
        op.add_argument("--ignoremissing", action="store_true", default=False,
                        help="""
                        Ignore any packages or groups specified in the packages
                        section that are not found in any configured repository.
                        The default behavior is to halt the installation and ask
                        the user if the installation should be aborted or
                        continued. This option allows fully automated
                        installation even in the error case.""",
                        introduced=FC4)
        op.add_argument("--nobase", action="store_true", default=False,
                        deprecated=F18, removed=F22, help="""
                        Do not install the @base group (installed by default,
                        otherwise).""")
        op.add_argument("--nocore", action="store_true", default=False,
                        introduced=F21, help="""
                        Do not install the @core group (installed by default,
                        otherwise).

                        **Omitting the core group can produce a system that is
                        not bootable or that cannot finish the install. Use
                        with caution.**""")
        op.add_argument("--ignoredeps", dest="resolveDeps", action="store_false",
                        deprecated=FC4, removed=F9, help="")
        op.add_argument("--resolvedeps", dest="resolveDeps", action="store_true",
                        deprecated=FC4, removed=F9, help="")
        op.add_argument("--default", dest="defaultPackages", action="store_true",
                        default=False, introduced=F7, help="""
                        Install the default package set. This corresponds to the
                        package set that would be installed if no other
                        selections were made on the package customization screen
                        during an interactive install.""")
        op.add_argument("--instLangs", default=None, introduced=F9, help="""
                        Specify the list of languages that should be installed.
                        This is different from the package group level
                        selections, though. This option does not specify what
                        package groups should be installed. Instead, it controls
                        which translation files from individual packages should
                        be installed by setting RPM macros.""")
        op.add_argument("--multilib", dest="multiLib", action="store_true",
                        default=False, introduced=F18, help="""
                        Enable yum's "all" multilib_policy as opposed to the
                        default of "best".""")
        op.add_argument("--excludeWeakdeps", dest="excludeWeakdeps",
                        action="store_true", default=False, introduced=F24,
                        help="""
                        Do not install packages from weak dependencies. These
                        are packages linked to the selected package set by
                        Recommends and Supplements flags. By default weak
                        dependencies will be installed.""")
        op.add_argument("--timeout", dest="timeout", type=int,
                        default=None, introduced=RHEL7, help="""
                        Set up yum's or dnf's timeout. It is a number of seconds
                        to wait for a connection before timing out.""")
        op.add_argument("--retries", dest="retries", type=int,
                        default=None, introduced=RHEL7, help="""
                        Set up yum's or dnf's retries. It is a number of times
                        any attempt to retrieve a file should retry before
                        returning an error.""")

        return op

    def handleHeader(self, lineno, args):
        """Process the arguments to the %packages header and set attributes
           on the Version's Packages instance appropriate.  This method may be
           overridden in a subclass if necessary.
        """
        Section.handleHeader(self, lineno, args)
        op = self._getParser()
        ns = op.parse_args(args=args[1:], lineno=lineno)

        if ns.defaultPackages and ns.nobase:
            raise KickstartParseError(_("--default and --nobase cannot be used together"), lineno=lineno)
        elif ns.defaultPackages and ns.nocore:
            raise KickstartParseError(_("--default and --nocore cannot be used together"), lineno=lineno)

        self.handler.packages.excludeDocs = ns.excludedocs
        self.handler.packages.addBase = not ns.nobase
        if ns.ignoremissing:
            self.handler.packages.handleMissing = KS_MISSING_IGNORE
        else:
            self.handler.packages.handleMissing = KS_MISSING_PROMPT

        if ns.defaultPackages:
            self.handler.packages.default = True

        if ns.instLangs is not None:
            self.handler.packages.instLangs = ns.instLangs

        self.handler.packages.nocore = ns.nocore
        self.handler.packages.multiLib = ns.multiLib
        self.handler.packages.excludeWeakdeps = ns.excludeWeakdeps
        self.handler.packages.timeout = ns.timeout
        self.handler.packages.retries = ns.retries
        self.handler.packages.seen = True



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2015 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.xconfig import FC3_XConfig, FC6_XConfig

class XConfig_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_XConfig()
        self.assertEqual(cmd.noProbe, False)
        self.assertEqual(cmd.startX, False)
        self.assertEqual(cmd.depth, 0)

        cmd.depth = -1
        self.assertTrue(cmd.__str__().find('--depth') > -1)

        cmd6 = FC6_XConfig()
        cmd6.depth = -1
        self.assertTrue(cmd6.__str__().find('--depth') > -1)


class FC3_TestCase(CommandTest):
    command = "xconfig"

    def runTest(self):
        # pass
        if "--card" in self.optionList:
            self.assert_parse("xconfig --card=cardA --hsync=H --vsync=V --monitor=monitorA --noprobe",
                              "xconfig  --card=cardA --hsync=H --monitor=monitorA --noprobe --vsync=V\n")

        if "--depth" in self.optionList:
            self.assert_parse("xconfig --depth=16 --resolution=1280x1024 --videoram=32000",
                              "xconfig  --depth=16 --resolution=1280x1024 --videoram=32000\n")

        self.assert_parse("xconfig --defaultdesktop=xfce --startxonboot",
                          "xconfig  --defaultdesktop=xfce --startxonboot\n")

        if "--server" in self.optionList:
            self.assert_parse("xconfig --server=Xvfb",
                              "xconfig  --server=Xvfb\n")

        # no arguments means don't print anything out
        self.assert_parse("xconfig", "")

        # fail
        # incorrect parameters
        self.assert_parse_error("xconfig --startxonboot=yes")

        # extra arguments
        self.assert_parse_error("xconfig --extra --arguments --here")
        self.assert_parse_error("xconfig extra arguments here")

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        if "--driver" in self.optionList:
            self.assert_parse("xconfig --driver=vesa",
                              "xconfig  --driver=vesa\n")

        # deprecated
        self.assert_deprecated("xconfig", "--card")
        self.assert_deprecated("xconfig", "--hsync")
        self.assert_deprecated("xconfig", "--monitor")
        self.assert_deprecated("xconfig", "--noprobe")
        self.assert_deprecated("xconfig", "--vsync")

        # removed
        self.assert_removed("xconfig", "--server")

class F9_TestCase(FC6_TestCase):
    def runTest(self):
        FC6_TestCase.runTest(self)

        self.assert_removed("xconfig", "--card")
        self.assert_removed("xconfig", "--hsync")
        self.assert_removed("xconfig", "--monitor")
        self.assert_removed("xconfig", "--noprobe")
        self.assert_removed("xconfig", "--vsync")

class F10_TestCase(F9_TestCase):
    def runTest(self):
        F9_TestCase.runTest(self)

        # deprecated
        self.assert_deprecated("xconfig", "--driver")
        self.assert_deprecated("xconfig", "--depth")
        self.assert_deprecated("xconfig", "--resolution")
        self.assert_deprecated("xconfig", "--videoram")

class F14_TestCase(F10_TestCase):
    def runTest(self):
        F10_TestCase.runTest(self)

        self.assert_removed("xconfig", "--driver")
        self.assert_removed("xconfig", "--depth")
        self.assert_removed("xconfig", "--resolution")
        self.assert_removed("xconfig", "--videoram")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent
from pykickstart.base import KickstartCommand
from pykickstart.version import FC3, F18, versionToLongString
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser, commaSplit

from pykickstart.i18n import _


class FC3_Keyboard(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.keyboard = kwargs.get("keyboard", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.keyboard:
            retval += "# System keyboard\nkeyboard %s\n" % self.keyboard

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="keyboard", description="""
            This required command sets system keyboard type.""", version=FC3)
        op.add_argument("kbd", nargs='*', help="Keyboard type", version=FC3)
        return op

    def parse(self, args):
        (_ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(_ns.kbd) != 1:
            raise KickstartParseError(_("Kickstart command %s requires one argument") % "keyboard", lineno=self.lineno)
        elif extra:
            mapping = {"command": "keyboard", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.keyboard = _ns.kbd[0]
        return self

class F18_Keyboard(FC3_Keyboard):
    def __init__(self, writePriority=0, *args, **kwargs):                # pylint: disable=super-init-not-called
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)  # pylint: disable=non-parent-init-called
        self.op = self._getParser()
        self._keyboard = kwargs.get("_keyboard", "")
        self.vc_keymap = kwargs.get("vc_keymap", "")
        self.x_layouts = kwargs.get("x_layouts", [])
        self.switch_options = kwargs.get("switch_options", [])

    def __str__(self):
        if not any((self._keyboard, self.x_layouts, self.vc_keymap)):
            return ""

        retval = "# Keyboard layouts\n"
        if not self.vc_keymap and not self.x_layouts:
            retval += "keyboard '%s'\n" % self._keyboard
            return retval

        if self._keyboard:
            retval += "# old format: keyboard %s\n" % self._keyboard
            retval += "# new format:\n"
        retval += "keyboard" + self._getArgsAsStr() + "\n"

        return retval

    def _getArgsAsStr(self):
        retval = ""

        if self.vc_keymap:
            retval += " --vckeymap=%s" % self.vc_keymap

        if self.x_layouts:
            layouts_str = "'%s'" % self.x_layouts[0]
            for layout in self.x_layouts[1:]:
                layouts_str += ",'%s'" % layout
            retval += " --xlayouts=%s" % layouts_str

        if self.switch_options:
            switch_str = "'%s'" % self.switch_options[0]
            for opt in self.switch_options[1:]:
                switch_str += ",'%s'" % opt
            retval += " --switch=%s" % switch_str

        return retval

    def _getParser(self):
        op = FC3_Keyboard._getParser(self)
        op.description += dedent("""

        .. versionchanged:: %s

        See the documentation of ``--vckeymap`` option and the tip at the end
        of this section for a guide how to get values accepted by this command.

        Either ``--vckeymap`` or ``--xlayouts`` must be used.

        Alternatively, use the older format, ``arg``, which is still supported.
        ``arg`` can be an X layout or VConsole keymap name.

        Missing values will be automatically converted from the given one(s).
        """ % versionToLongString(F18))

        op.add_argument("--vckeymap", dest="vc_keymap", default="", help="""
                        Specify VConsole keymap that should be used. is a keymap
                        name which is the same as the filename under
                        ``/usr/lib/kbd/keymaps/`` without the ``.map.gz`` extension.
                        """, version=F18)
        op.add_argument("--xlayouts", dest="x_layouts", type=commaSplit,
                        version=F18, help="""
                        Specify a list of X layouts that should be used
                        (comma-separated list without spaces). Accepts the same
                        values as ``setxkbmap(1)``, but uses either the layout format
                        (such as cz) or the 'layout (variant)' format (such as
                        'cz (qwerty)'). For example::

                        ``keyboard --xlayouts=cz,'cz (qwerty)'`""")
        op.add_argument("--switch", dest="switch_options", type=commaSplit,
                        version=F18, help="""
                        Specify a list of layout switching options that should
                        be used (comma-separated list without spaces). Accepts
                        the same values as ``setxkbmap(1)`` for layout switching.
                        For example::

                        ``keyboard --xlayouts=cz,'cz (qwerty)' --switch=grp:alt_shift_toggle``
                        """)
        op.epilog = dedent("""
        *If you know only the description of the layout (e.g. Czech (qwerty)),
        you can use http://vpodzime.fedorapeople.org/layouts_list.py to list
        all available layouts and find the one you want to use. The string in
        square brackets is the valid layout specification as Anaconda accepts
        it. The same goes for switching options and
        http://vpodzime.fedorapeople.org/switching_list.py*""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if len(ns.kbd) > 1:
            message = _("A single argument is expected for the %s command") % "keyboard"
            raise KickstartParseError(message, lineno=self.lineno)
        elif extra:
            mapping = {"command": "keyboard", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)
        elif not any([ns.kbd, self.vc_keymap, self.x_layouts]):
            message = _("One of --xlayouts, --vckeymap options with value(s) "
                        "or argument is expected for the keyboard command")
            raise KickstartParseError(message, lineno=self.lineno)

        if ns.kbd:
            self._keyboard = ns.kbd[0]

        return self

    # property for backwards compatibility
    # pylint: disable=method-hidden
    @property
    def keyboard(self):
        if self.x_layouts:
            return self._keyboard or self.vc_keymap or self.x_layouts[0]
        else:
            return self._keyboard or self.vc_keymap or ""

    # pylint: disable=function-redefined,no-member
    @keyboard.setter
    def keyboard(self, value):
        self._keyboard = value



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.autopart import FC3_AutoPart, RHEL6_AutoPart
from pykickstart.version import RHEL6, RHEL7, F21, F23

class AutoPart_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_AutoPart()
        self.assertEqual(cmd.__str__(), '')

class FC3_TestCase(CommandTest):
    command = "autopart"

    def runTest(self):
        # pass
        self.assert_parse("autopart", "autopart\n")

        # fail - on FC3, autopart  took no options so this raises a different
        # exception than later releases.
        if self.__class__.__name__ == "FC3_TestCase":
            self.assert_parse_error("autopart --blah")

class F9_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --passphrase=whatever", "autopart\n")
        self.assert_parse("autopart --encrypted", "autopart --encrypted\n")
        self.assert_parse("autopart --encrypted --passphrase=\"whatever\"",
                          "autopart --encrypted --passphrase=\"whatever\"\n")
        self.assert_parse("autopart --encrypted --passphrase=whatever",
                          "autopart --encrypted --passphrase=\"whatever\"\n")

        # fail
        self.assert_parse_error("autopart --passphrase")
        self.assert_parse_error("autopart --encrypted --passphrase")
        self.assert_parse_error("autopart --encrypted=False")
        self.assert_parse_error("autopart --encrypted=True")

class F12_TestCase(F9_TestCase):
    def runTest(self):
        # Run F9 test case
        F9_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --escrowcert=\"http://x/y\"", "autopart\n")
        self.assert_parse("autopart --encrypted --backuppassphrase",
                          "autopart --encrypted\n")
        self.assert_parse("autopart --encrypted --escrowcert=\"http://x/y\"",
                          "autopart --encrypted --escrowcert=\"http://x/y\"\n")
        self.assert_parse("autopart --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase",
                          "autopart --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase\n")
        self.assert_parse("autopart --encrypted --escrowcert=http://x/y",
                          "autopart --encrypted --escrowcert=\"http://x/y\"\n")

        # fail
        self.assert_parse_error("autopart --escrowcert")
        self.assert_parse_error("autopart --escrowcert --backuppassphrase")
        self.assert_parse_error("autopart --encrypted --escrowcert "
                                "--backuppassphrase")
        self.assert_parse_error("autopart --backuppassphrase=False")
        self.assert_parse_error("autopart --backuppassphrase=True")

class RHEL6_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --cipher=foo", "autopart\n")
        self.assert_parse("autopart --encrypted --cipher=3-rot13",
                          "autopart --encrypted --cipher=\"3-rot13\"\n")

        # fail
        self.assert_parse_error("autopart --cipher")
        self.assert_parse_error("autopart --encrypted --cipher")

        # extra test coverage
        cmd = RHEL6_AutoPart()
        self.assertEqual(cmd.__str__(), '')

class RHEL6_Conflict_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = RHEL6

    def runTest(self):
        self.assert_parse_error("""
part / --size=1024 --fstype=ext4
autopart""")

        self.assert_parse_error("""
raid / --device=md0 --level=0 raid.01
autopart""")

        self.assert_parse_error("""
volgroup vg.01 pv.01 --pesize=70000
autopart""")

        self.assert_parse_error("""
logvol / --size=10240 --name=NAME --vgname=VGNAM
autopart""")


class F16_TestCase(F12_TestCase):
    def runTest(self):
        # Run F12 test case
        F12_TestCase.runTest(self)

        if "--type" not in self.optionList:
            # pass
            self.assert_parse("autopart --nolvm",
                              "autopart --nolvm\n")

            # fail
            self.assert_parse_error("autopart --nolvm=asdf")
            self.assert_parse_error("autopart --nolvm True")
            self.assert_parse_error("autopart --nolvm=1")
            self.assert_parse_error("autopart --nolvm 0")

class F17_TestCase(F16_TestCase):
    def runTest(self):
        # Run F16 test case
        F16_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --type=plain",
                          "autopart --type=plain\n")
        self.assert_parse("autopart --type=partition",
                          "autopart --type=plain\n")
        self.assert_parse("autopart --type=lvm",
                          "autopart --type=lvm\n")

        if self.__class__.__name__ != "RHEL8_TestCase":
            self.assert_parse("autopart --type=btrfs",
                              "autopart --type=btrfs\n")

        self.assert_parse("autopart --nolvm",
                          "autopart --type=plain\n")

        # don't add --type= if none was specified
        self.assert_parse("autopart",
                          "autopart\n")

        # fail
        self.assert_parse_error("autopart --type")
        self.assert_parse_error("autopart --type=blah")

class F18_TestCase(F17_TestCase):
    def runTest(self):
        F17_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --cipher=foo", "autopart\n")
        self.assert_parse("autopart --encrypted --cipher=3-rot13",
                          "autopart --encrypted --cipher=\"3-rot13\"\n")

        # fail
        self.assert_parse_error("autopart --cipher")
        self.assert_parse_error("autopart --encrypted --cipher")

class F20_TestCase(F18_TestCase):
    def runTest(self):
        F18_TestCase.runTest(self)

        self.assert_parse("autopart --type=thinp",
                          "autopart --type=thinp\n")

class F21_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --fstype=ext4",
                          'autopart --fstype=ext4\n')
        self.assert_parse("autopart --encrypted --fstype=ext4",
                          'autopart --encrypted --fstype=ext4\n')
        self.assert_parse("autopart --type=lvm --fstype=xfs",
                          "autopart --type=lvm --fstype=xfs\n")

        # fail
        self.assert_parse_error("autopart --fstype")
        self.assert_parse_error("autopart --fstype=btrfs")
        self.assert_parse_error("autopart --type=btrfs --fstype=xfs")


class F21_Conflict_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F21

    def runTest(self):
        self.assert_parse_error("""
part / --size=1024 --fstype=ext4
autopart""")

        self.assert_parse_error("""
raid / --device=md0 --level=0 raid.01
autopart""")

        self.assert_parse_error("""
volgroup vg.01 pv.01 --pesize=70000
autopart""")

        self.assert_parse_error("""
logvol / --size=10240 --name=NAME --vgname=VGNAM
autopart""")


class F23_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

class F23_Conflict_TestCase(F21_Conflict_TestCase):
    """
        reqpart was added in Fedora 23 but the autopart handler
        wasn't updated. Instead the changes were introduced into
        F20_AutoPart, which wasn't the best thing to do.
    """
    def __init__(self, *args, **kwargs):
        F21_Conflict_TestCase.__init__(self, *args, **kwargs)
        self.version = F23

    def runTest(self):
        F21_Conflict_TestCase.runTest(self)

        self.assert_parse_error("""
reqpart
autopart""")


class RHEL7_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --nohome",
                          "autopart --nohome\n")

        # fail
        self.assert_parse_error("autopart --nohome=xxx")
        self.assert_parse_error("autopart --nohome True")
        self.assert_parse_error("autopart --nohome=1")
        self.assert_parse_error("autopart --nohome 0")

class RHEL7_Conflict_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = RHEL7

    def runTest(self):
        self.assert_parse_error("""
reqpart
autopart""")


class F26_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

        # pass
        self.assert_parse("autopart --nohome",
                          'autopart --nohome\n')
        self.assert_parse("autopart --noboot",
                          'autopart --noboot\n')
        self.assert_parse("autopart --noswap",
                          'autopart --noswap\n')
        self.assert_parse("autopart --nohome --noboot --noswap",
                          'autopart --nohome --noboot --noswap\n')

        # fail
        self.assert_parse_error("autopart --nohome=xxx")
        self.assert_parse_error("autopart --nohome True")
        self.assert_parse_error("autopart --nohome=1")
        self.assert_parse_error("autopart --nohome 0")

        self.assert_parse_error("autopart --noboot=xxx")
        self.assert_parse_error("autopart --noboot True")
        self.assert_parse_error("autopart --noboot=1")
        self.assert_parse_error("autopart --noboot 0")

        self.assert_parse_error("autopart --noswap=xxx")
        self.assert_parse_error("autopart --noswap True")
        self.assert_parse_error("autopart --noswap=1")
        self.assert_parse_error("autopart --noswap 0")

class F26_Conflict_TestCase(F23_Conflict_TestCase):
    def runTest(self):
        F23_Conflict_TestCase.runTest(self)

class F29_TestCase(F26_TestCase):
    def runTest(self):
        F26_TestCase.runTest(self)

        self.assert_parse("autopart --encrypted --luks-version=luks2",
                          "autopart --encrypted --luks-version=luks2\n")

        self.assert_parse("autopart --encrypted --pbkdf=argon2i",
                          "autopart --encrypted --pbkdf=argon2i\n")

        self.assert_parse("autopart --encrypted --pbkdf-memory=256",
                          "autopart --encrypted --pbkdf-memory=256\n")

        self.assert_parse("autopart --encrypted --pbkdf-time=100",
                          "autopart --encrypted --pbkdf-time=100\n")

        self.assert_parse("autopart --encrypted --pbkdf-iterations=1000",
                          "autopart --encrypted --pbkdf-iterations=1000\n")

        self.assert_parse_error("autopart --encrypted --pbkdf-time=100 --pbkdf-iterations=1000")

class RHEL8_TestCase(F29_TestCase):
    def runTest(self):
        F29_TestCase.runTest(self)
        self.assert_parse_error("autopart --type=btrfs")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F13, F14, F18, RHEL7, F27, RHEL8
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_Url(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.url = kwargs.get("url", None)

        self.op = self._getParser()

    def __eq__(self, other):
        if not other:
            return False

        return self.url == other.url

    def __ne__(self, other):
        return not self == other

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.seen:
            retval += "# Use network installation\nurl --url=\"%s\"\n" % self.url

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="url", description="""
                            Install from an installation tree on a remote server
                            via FTP or HTTP.""", version=FC3)
        op.add_argument("--url", required=True, version=FC3, help="""
                        The URL to install from. Variable substitution is done
                        for $releasever and $basearch in the url.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F13_Url(FC3_Url):
    removedKeywords = FC3_Url.removedKeywords
    removedAttrs = FC3_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        FC3_Url.__init__(self, *args, **kwargs)
        self.proxy = kwargs.get("proxy", "")

    def __eq__(self, other):
        if not FC3_Url.__eq__(self, other):
            return False

        return self.proxy == other.proxy

    def __str__(self):
        retval = FC3_Url.__str__(self)

        if self.seen and self.proxy:
            retval = retval.strip()
            retval += " --proxy=\"%s\"\n" % self.proxy

        return retval

    def _getParser(self):
        op = FC3_Url._getParser(self)
        op.add_argument("--proxy", metavar="URL", version=F13,
                        help="""
                        Specify an HTTP/HTTPS/FTP proxy to use while performing
                        the install. The various parts of the argument act like
                        you would expect. The syntax is::

                            [protocol://][username[:password]@]host[:port]
                        """)
        return op

class F14_Url(F13_Url):
    removedKeywords = F13_Url.removedKeywords
    removedAttrs = F13_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_Url.__init__(self, *args, **kwargs)
        self.noverifyssl = kwargs.get("noverifyssl", False)

    def __eq__(self, other):
        if not F13_Url.__eq__(self, other):
            return False

        return self.noverifyssl == other.noverifyssl

    def __str__(self):
        retval = F13_Url.__str__(self)

        if self.seen and self.noverifyssl:
            retval = retval.strip()
            retval += " --noverifyssl\n"

        return retval

    def _getParser(self):
        op = F13_Url._getParser(self)
        op.add_argument("--noverifyssl", action="store_true", version=F14,
                        default=False, help="""
                        For a tree on a HTTPS server do not check the server's
                        certificate with what well-known CA validate and do not
                        check the server's hostname matches the certificate's
                        domain name.""")
        return op

class RHEL6_Url(F14_Url):
    pass

class F18_Url(F14_Url):
    removedKeywords = F14_Url.removedKeywords
    removedAttrs = F14_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        F14_Url.__init__(self, *args, **kwargs)
        self.mirrorlist = kwargs.get("mirrorlist", None)
        self.exclusive_required_options = [("mirrorlist", "--mirrorlist"),
                                           ("url", "--url")]

    def __eq__(self, other):
        if not F14_Url.__eq__(self, other):
            return False

        return self.mirrorlist == other.mirrorlist

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use network installation\n"

        if self.url:
            retval += "url --url=\"%s\"" % self.url
        elif self.mirrorlist:
            retval += "url --mirrorlist=\"%s\"" % self.mirrorlist

        if self.proxy:
            retval += " --proxy=\"%s\"" % self.proxy

        if self.noverifyssl:
            retval += " --noverifyssl"

        return retval + "\n"

    def _getParser(self):
        op = F14_Url._getParser(self)
        op.add_argument("--url", version=F18, help="""
                        This parameter is no longer required because you could
                        use ``--mirrorlist`` instead.""")
        op.add_argument("--mirrorlist", metavar="URL", version=F18, help="""
                        The mirror URL to install from. Variable substitution
                        is done for $releasever and $basearch in the url.""")
        return op

    def parse(self, args):
        retval = F14_Url.parse(self, args)

        ns = self.op.parse_args(args=args, lineno=self.lineno)
        # Check that just one of exclusive required options is specified
        used_options = [opt for attr, opt in self.exclusive_required_options
                        if getattr(ns, attr, None)]
        if len(used_options) == 0:
            mapping = {"options_list": ", ".join((opt for attr, opt in self.exclusive_required_options))}
            raise KickstartParseError(_("One of -%(options_list)s options must be specified for url command.") % mapping, lineno=self.lineno)
        if len(used_options) > 1:
            mapping = {"options_list": ", ".join((opt for opt in used_options))}
            raise KickstartParseError(_("Only one of %(options_list)s options may be specified for url command.") % mapping, lineno=self.lineno)

        return retval

class RHEL7_Url(F18_Url):
    removedKeywords = F18_Url.removedKeywords
    removedAttrs = F18_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        F18_Url.__init__(self, *args, **kwargs)
        self.sslcacert = kwargs.get("sslcacert", None)
        self.sslclientcert = kwargs.get("sslclientcert", None)
        self.sslclientkey = kwargs.get("sslclientkey", None)

    def __str__(self):
        retval = F18_Url.__str__(self)
        if not self.seen:
            return retval

        retval = retval[:-1]  # strip '\n'

        if self.sslcacert:
            retval += " --sslcacert=\"%s\"" % self.sslcacert

        if self.sslclientcert:
            retval += " --sslclientcert=\"%s\"" % self.sslclientcert

        if self.sslclientkey:
            retval += " --sslclientkey=\"%s\"" % self.sslclientkey

        return retval + "\n"

    def _getParser(self):
        op = F18_Url._getParser(self)
        op.add_argument("--sslcacert", version=RHEL7, help="""
                        Path to the file holding one or more SSL certificates
                        to verify the repository host with.""")
        op.add_argument("--sslclientcert", version=RHEL7, help="""
                        Path to the SSL client certificate (PEM file) which
                        should be used to connect to the repository.""")
        op.add_argument("--sslclientkey", version=RHEL7, help="""
                        Path to the private key file associated with the client
                        certificate given with --sslclientcert.""")
        return op

class F27_Url(F18_Url):
    removedKeywords = F18_Url.removedKeywords
    removedAttrs = F18_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        F18_Url.__init__(self, *args, **kwargs)
        self.metalink = kwargs.get("metalink", None)
        self.exclusive_required_options.append(("metalink", "--metalink"))

    def __eq__(self, other):
        if not F18_Url.__eq__(self, other):
            return False

        return self.metalink == other.metalink

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use network installation\n"

        if self.url:
            retval += "url --url=\"%s\"" % self.url
        elif self.mirrorlist:
            retval += "url --mirrorlist=\"%s\"" % self.mirrorlist
        elif self.metalink:
            retval += "url --metalink=\"%s\"" % self.metalink

        if self.proxy:
            retval += " --proxy=\"%s\"" % self.proxy

        if self.noverifyssl:
            retval += " --noverifyssl"

        return retval + "\n"

    def _getParser(self):
        op = F18_Url._getParser(self)
        op.add_argument("--metalink", metavar="URL", version=F27, help="""
                        The metalink URL to install from. Variable substitution
                        is done for $releasever and $basearch in the url.""")
        return op

class RHEL8_Url(F27_Url):
    removedKeywords = F27_Url.removedKeywords
    removedAttrs = F27_Url.removedAttrs

    def __init__(self, *args, **kwargs):
        F27_Url.__init__(self, *args, **kwargs)
        self.sslcacert = kwargs.get("sslcacert", None)
        self.sslclientcert = kwargs.get("sslclientcert", None)
        self.sslclientkey = kwargs.get("sslclientkey", None)

    def __str__(self):
        retval = F27_Url.__str__(self)
        if not self.seen:
            return retval

        retval = retval[:-1]  # strip '\n'

        if self.sslcacert:
            retval += " --sslcacert=\"%s\"" % self.sslcacert

        if self.sslclientcert:
            retval += " --sslclientcert=\"%s\"" % self.sslclientcert

        if self.sslclientkey:
            retval += " --sslclientkey=\"%s\"" % self.sslclientkey

        return retval + "\n"

    def _getParser(self):
        op = F27_Url._getParser(self)
        op.add_argument("--sslcacert", version=RHEL8, help="""
                        Path to the file holding one or more SSL certificates
                        to verify the repository host with.""")
        op.add_argument("--sslclientcert", version=RHEL8, help="""
                        Path to the SSL client certificate (PEM file) which
                        should be used to connect to the repository.""")
        op.add_argument("--sslclientkey", version=RHEL8, help="""
                        Path to the private key file associated with the client
                        certificate given with --sslclientcert.""")
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import warnings

from pykickstart.errors import KickstartDeprecationWarning
from pykickstart.version import FC3
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_ZeroMbr(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=110, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.zerombr = kwargs.get("zerombr", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.zerombr:
            retval += "# Clear the Master Boot Record\nzerombr\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="zerombr", description="""
                            If zerombr is specified, any disks whose formatting
                            is unrecognized are initialized. This will destroy
                            all of the contents of disks with invalid partition
                            tables or other formatting unrecognizable to the
                            installer. It is useful so that the installation
                            program does not ask if it should initialize the
                            disk label if installing to a brand new hard drive.
                            """, version=FC3)
        return op

    def parse(self, args):
        extra = self.op.parse_known_args(args=args, lineno=self.lineno)[1]

        if extra:
            warnings.warn(_("Ignoring deprecated option on line %s:  The zerombr command no longer takes any options.  In future releases, this will result in a fatal error from kickstart.  Please modify your kickstart file to remove any options.") % self.lineno, KickstartDeprecationWarning)

        self.zerombr = True
        return self

class F9_ZeroMbr(FC3_ZeroMbr):
    removedKeywords = FC3_ZeroMbr.removedKeywords
    removedAttrs = FC3_ZeroMbr.removedAttrs

    def parse(self, args):
        self.op.parse_args(args=args, lineno=self.lineno)
        self.zerombr = True
        return self



#
# Martin Kolman <mkolman@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import F29, RHEL8

from pykickstart.i18n import _


class F29_ModuleData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.name = kwargs.get("name", "")
        self.stream = kwargs.get("stream", "")

    def __eq__(self, y):
        if not y:
            return False
        return (self.name == y.name and self.stream == y.stream)

    def __ne__(self, y):
        return not self == y


    def __str__(self):
        retval = BaseData.__str__(self)

        args = self._getArgsAsStr()
        if args:
            retval += "module%s\n" % args

        return retval

    def _getArgsAsStr(self):
        retval = " --name=%s" % self.name
        # --stream is optional
        if self.stream:
            retval += " --stream=%s" % self.stream
        return retval

class RHEL8_ModuleData(F29_ModuleData):
    removedKeywords = F29_ModuleData.removedKeywords
    removedAttrs = F29_ModuleData.removedAttrs

    def __init__(self, *args, **kwargs):
        F29_ModuleData.__init__(self, *args, **kwargs)
        self.enable = kwargs.get("enable", True)

    def _getArgsAsStr(self):
        retval = F29_ModuleData._getArgsAsStr(self)

        # --disable is optional
        if not self.enable:
            retval += " --disable"

        return retval


class F29_Module(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

        self.moduleList = kwargs.get("moduleList", [])
        self.op = self._getParser()

    def __str__(self):
        retval = ""

        for module in self.moduleList:
            retval += module.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="module", description="""
                            The module command makes it possible to manipulate
                            modules.

                            (In this case we mean modules as introduced by the
                            Fedora modularity initiative.)

                            A module is defined by a unique name and a stream id,
                            where single module can (and usually has) multiple
                            available streams.

                            Streams will in most cases corresponds to stable
                            releases of the given software components
                            (such as Node.js, Django, etc.) but there could be
                            also other use cases, such as a raw upstream master
                            branch stream or streams corresponding to an upcoming
                            stable release.

                            For more information see the Fedora modularity
                            initiative documentation:
                            https://docs.pagure.org/modularity/""", version=F29)
        op.add_argument("--name", metavar="<module_name>", version=F29, required=True,
                        help="""
                        Name of the module to enable.""")
        op.add_argument("--stream", metavar="<module_stream_name>", version=F29, required=False,
                        help="""
                        Name of the module stream to enable.""")

        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) > 0:
            msg = _("The enable module command does not take position arguments!")
            raise KickstartParseError(msg, lineno=self.lineno)

        enable_module_data = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, enable_module_data)
        enable_module_data.lineno = self.lineno

        return enable_module_data

    def dataList(self):
        return self.moduleList

    @property
    def dataClass(self):
        return self.handler.ModuleData

class RHEL8_Module(F29_Module):
    removedKeywords = F29_Module.removedKeywords
    removedAttrs = F29_Module.removedAttrs

    def _getParser(self):
        op = F29_Module._getParser(self)
        op.add_argument("--disable", version=RHEL8,
                        action="store_false",
                        dest="enable",
                        help="Disable module.")
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, RHEL8, versionToLongString
from pykickstart.base import KickstartCommand, DeprecatedCommand
from pykickstart.options import KSOptionParser

class FC3_AutoStep(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.autostep = kwargs.get("autostep", False)
        self.autoscreenshot = kwargs.get("autoscreenshot", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.autostep:
            if self.autoscreenshot:
                retval += "autostep --autoscreenshot\n"
            else:
                retval += "autostep\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="autostep", description="""
                            Kickstart installs normally skip unnecessary screens.
                            This makes the installer step through every screen,
                            displaying each briefly.

                            This is mostly used for debugging.""",
                            version=FC3)
        op.add_argument("--autoscreenshot", action="store_true", default=False,
                        version=FC3, help="""
                        Take a screenshot at every step during installation and
                        copy the images over to ``/root/anaconda-screenshots`` after
                        installation is complete. This is most useful for
                        documentation.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        self.autostep = True
        return self

class RHEL8_AutoStep(DeprecatedCommand, FC3_AutoStep):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC3_AutoStep._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(RHEL8)
        return op



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.nfs import FC3_NFS

class NFS_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = FC3_NFS()
        data2 = FC3_NFS()

        # extra test coverage
        self.assertEqual(data1.__str__(), '')

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['server', 'dir']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class FC3_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install", "nfs --server=1.2.3.4 --dir=/install\n")

        self.assertFalse(self.assert_parse("nfs --server=1.2.3.4 --dir=/install") is None)
        self.assertTrue(self.assert_parse("nfs --server=1.2.3.4 --dir=/install") !=
                        self.assert_parse("nfs --server=2.3.4.5 --dir=/install"))
        self.assertFalse(self.assert_parse("nfs --server=1.2.3.4 --dir=/install") ==
                         self.assert_parse("nfs --server=2.3.4.5 --dir=/install"))
        self.assertFalse(self.assert_parse("nfs --server=1.2.3.4 --dir=/install") ==
                         self.assert_parse("nfs --server=1.2.3.4 --dir=/install2"))

        # fail
        # missing required options --server and --dir
        self.assert_parse_error("nfs")
        self.assert_parse_error("nfs --server=1.2.3.4")
        self.assert_parse_error("nfs --server")
        self.assert_parse_error("nfs --dir=/install")
        self.assert_parse_error("nfs --dir")
        # unknown option
        self.assert_parse_error("nfs --unknown=value")

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options", "nfs --server=1.2.3.4 --dir=/install --opts=\"options\"\n")

        self.assertTrue(self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options") ==
                        self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options"))
        self.assertFalse(self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options") ==
                         self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=other,options"))

        # fail
        # --opts requires argument if specified
        self.assert_parse_error("nfs --server=1.2.3.4 --dir=/install --opts")

if __name__ == "__main__":
    unittest.main()



import unittest
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.realm import F19_Realm
from pykickstart.version import F19

class Realm_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = F19_Realm()
        # additional code coverage
        self.assertEqual(cmd.__str__(), "")

class F19_TestCase(CommandTest):
    command = "realm"

    def runTest(self):

        # No realm command arguments
        self.assert_parse_error("realm")

        # Unsupported realmcommand
        self.assert_parse_error("realm unknown --args")

        # pass for join
        realm = self.assert_parse("realm join blah")
        self.assertEqual(realm.join_realm, "blah")
        self.assertEqual(realm.join_args, ["blah"])
        self.assertEqual(realm.discover_options, [])
        self.assertEqual(str(realm), "# Realm or domain membership\nrealm join blah\n")

        # pass for join with client-software
        realm = self.assert_parse("realm join --client-software=sssd --computer-ou=OU=blah domain.example.com")
        self.assertEqual(realm.join_realm, "domain.example.com")
        self.assertEqual(realm.join_args, ["--client-software=sssd", "--computer-ou=OU=blah", "domain.example.com"])
        self.assertEqual(realm.discover_options, ["--client-software=sssd"])
        self.assertEqual(str(realm), "# Realm or domain membership\nrealm join --client-software=sssd --computer-ou=OU=blah domain.example.com\n")

        # pass for join with one-time password
        realm = self.assert_parse("realm join --one-time-password=12345 domain.example.com")

        # pass for join with no password
        realm = self.assert_parse("realm join --no-password domain.example.com")

        # Bad arguments, the --no-password does not support an argument
        self.assert_parse_error("realm join --no-password=blah one.example.com")

        # Bad arguments, only one domain for join
        self.assert_parse_error("realm join one two")

        # Bad arguments, unsupported argument
        self.assert_parse_error("realm join --user=blah one.example.com")

        # Domain must be the last argument
        self.assert_parse_error("realm join test_domain --no-password")

class F19_MultipleJoin_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F19

    def runTest(self):
        # fail - can't use join more than once
        self.assert_parse_error("""
realm join blah1
realm join blah2""")

if __name__ == "__main__":
    unittest.main()



from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandSequenceTest


class AutopartUsage_TestCase(CommandSequenceTest):
    """Test autopart command usage

    Autopart can't be used together with any of the part/partition, raid,
    logvol and volgroup command in the same kickstart file.
    This test case is used to check if parsing of such kickstarts correctly
    raises a parsing error.
    """

    def runTest(self):
        # first just parse some correct sequences to
        # check if the parsing itself works correctly
        correct_command_sequences = [
            "part / --size=2048",
            "partition / --size=2048",
            "autopart",
            "raid / --level=1 --device=md0 raid.01",
            "logvol / --vgname=foo --size=2000 --name=bar",
            "volgroup foo pv.01"
        ]
        for sequence in correct_command_sequences:
            self.assert_parse(sequence)

        # then check various incorrect sequences
        incorrect_command_sequences = [
            "part / --size=2048\nautopart",
            "autopart\npart / --size=2048",
            "partition / --size=2048\nautopart",
            "autopart\npartition / --size=2048",
            "raid / --level=1 --device=md0 raid.01\nautopart",
            "autopart\nraid / --level=1 --device=md0 raid.01",
            "logvol / --vgname=foo --size=2000 --name=bar\nautopart",
            "autopart\nlogvol / --vgname=foo --size=2000 --name=bar",
            "volgroup foo pv.01\nautopart",
            "autopart\nvolgroup foo pv.01"
        ]

        for sequence in incorrect_command_sequences:
            self.assert_parse_error(sequence)

        # also test all the commands being used at once

        long_incorrect_sequence = """
part / --size=2048
partition /opt --size=2048
autopart
raid / --level=1 --device=md0 raid.01
logvol / --vgname=foo --size=2000 --name=bar
volgroup foo pv.01
"""
        self.assert_parse_error(long_incorrect_sequence)


class TmpfsUsage_TestCase(CommandSequenceTest):
    """Test tmpfs usage with the partition command"""

    def runTest(self):
        # the tmpfs filesystem supports the size and fsopt options
        self.assert_parse('part /foo --size=100 --fstype=tmpfs --fsoptions="noexec"')
        self.assert_parse('part /ham --fstype=tmpfs --fsoptions="size=250%"')
        self.assert_parse('part /tmp --size=20000 --fstype=tmpfs')

        # the tmpfs filesystem dos not support the --grow and --maxsize
        # options
        self.assert_parse_error("part --fstype=tmpfs /tmp --grow")
        self.assert_parse_error("part --fstype=tmpfs /tmp --grow --maxsize=1000")
        self.assert_parse_error("part --fstype=tmpfs /tmp --maxsize=1000")

class PartitionDupes_TestCase(CommandSequenceTest):
    def runTest(self):
        # Two swap partitions is alright.
        self.assert_parse("""
part swap --size=1024 --fstype=swap
part swap --size=2048 --fstype=swap""")

        # Two root partitions is not.
        self.assert_parse_error("""
part / --size=1024 --fstype=ext4
part / --size=2048 --fstype=ext4""", exception=KickstartParseWarning)
